@@ -1,6 +1,6 @@
 #!/usr/bin/env python
-#emacs: -*- mode: python-mode; py-indent-offset: 4; indent-tabs-mode: nil -*-
-#ex: set sts=4 ts=4 sw=4 et:
+# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
+# vi: set ft=python sts=4 ts=4 sw=4 et:
 ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ##
 #
 #   See COPYING file distributed along with the PyMVPA package for the
@@ -16,13 +16,13 @@ plot various time-locked data types. This example uses MEG data and therefore
 generates an ERF-plot.
 """
 
-from mvpa.suite import *
+from mvpa2.suite import *
 
 # load data
-meg = TuebingenMEG(os.path.join('data', 'tueb_meg.dat.gz'))
+meg = TuebingenMEG(os.path.join(pymvpa_dataroot, 'tueb_meg.dat.gz'))
 
 
-# Define plots for easy feeding into plotERP
+# Define plots for easy feeding into plot_erp
 plots = []
 colors = ['r', 'b', 'g']
 
@@ -49,7 +49,7 @@ cargs = {
     }
 
 # Create a new figure
-fig = P.figure(figsize=(12, 8))
+fig = pl.figure(figsize=(12, 8))
 
 # Following plots are plotted inverted (negative up) for the
 # demonstration of this capability and elderly convention for ERP
@@ -59,30 +59,30 @@ fig = P.figure(figsize=(12, 8))
 # Plot MEG sensors
 
 # frame_on=False guarantees abent outside rectangular axis with
-# labels. plotERP recreates its own axes centered at (0,0)
+# labels. plot_erp recreates its own axes centered at (0,0)
 ax = fig.add_subplot(2, 1, 1, frame_on=False)
-plotERPs(plots[:2], ylabel='$pT$', ymult=-1e12, ax=ax, **cargs)
+plot_erps(plots[:2], ylabel='$pT$', ymult=-1e12, ax=ax, **cargs)
 
 # Plot EEG sensor
 ax = fig.add_subplot(2, 1, 2, frame_on=False)
-plotERPs(plots[2:3], ax=ax, ymult=-1e6, **cargs)
+plot_erps(plots[2:3], ax=ax, ymult=-1e6, **cargs)
 
 # Additional example: plotting a single ERP on an existing plot
 # without drawing axis:
 #
-# plotERP(data=meg.data[:, 0, :], SR=meg.samplingrate, pre=pre,
+# plot_erp(data=meg.data[:, 0, :], SR=meg.samplingrate, pre=pre,
 #         pre_mean=pre, errtype=errtype, ymult=-1.0)
 
 if cfg.getboolean('examples', 'interactive', True):
     # show all the cool figures
-    P.show()
+    pl.show()
 
 """
 The ouput of the provided example is presented below. It is not a very
 fascinating one due to the limited number of samples provided in the
 dataset shipped within the toolbox.
 
-.. image:: ../pics/erp_plot.*
+.. image:: ../pics/ex_erp_plot.*
    :align: center
    :alt: ERP plot example
 
