@@ -14,26 +14,29 @@ Kernel-Demo
 This is an example demonstrating various kernel implementation in PyMVPA.
 """
 
-from mvpa.suite import *
-from mvpa.clfs.kernel import *
-import pylab as P
+import numpy as np
+from mvpa2.support.pylab import pl
+
+#from mvpa2.suite import *
+from mvpa2.base import cfg
+from mvpa2.kernels.np import *
 
 
-# N.random.seed(1)
-data = N.random.rand(4, 2)
+# np.random.seed(1)
+data = np.random.rand(4, 2)
 
 for kernel_class, kernel_args in (
-    (KernelConstant, {'sigma_0':1.0}),
-    (KernelConstant, {'sigma_0':1.0}),
-    (KernelLinear, {'Sigma_p':N.eye(data.shape[1])}),
-    (KernelLinear, {'Sigma_p':N.ones(data.shape[1])}),
-    (KernelLinear, {'Sigma_p':2.0}),
-    (KernelLinear, {}),
-    (KernelExponential, {}),
-    (KernelSquaredExponential, {}),
-    (KernelMatern_3_2, {}),
-    (KernelMatern_5_2, {}),
-    (KernelRationalQuadratic, {}),
+    (ConstantKernel, {'sigma_0':1.0}),
+    (ConstantKernel, {'sigma_0':1.0}),
+    (GeneralizedLinearKernel, {'Sigma_p':np.eye(data.shape[1])}),
+    (GeneralizedLinearKernel, {'Sigma_p':np.ones(data.shape[1])}),
+    (GeneralizedLinearKernel, {'Sigma_p':2.0}),
+    (GeneralizedLinearKernel, {}),
+    (ExponentialKernel, {}),
+    (SquaredExponentialKernel, {}),
+    (Matern_3_2Kernel, {}),
+    (Matern_5_2Kernel, {}),
+    (RationalQuadraticKernel, {}),
     ):
     kernel = kernel_class(**kernel_args)
     print kernel
@@ -47,21 +50,23 @@ for kernel_class, kernel_args in (
 # used for regression/classfication with GPR/GPC.
 count = 1
 for k in kernel_dictionary.keys():
-    P.subplot(3,4,count)
-    # X = N.random.rand(size)*12.0-6.0
+    pl.subplot(3, 4, count)
+    # X = np.random.rand(size)*12.0-6.0
     # X.sort()
-    X = N.arange(-1,1,.02)
-    X = X[:,N.newaxis]
+    X = np.arange(-1, 1, .02)
+    X = X[:, np.newaxis]
     ker = kernel_dictionary[k]()
-    K = ker.compute(X,X)
+    ker.compute(X, X)
+    print k
+    K = np.asarray(ker)
     for i in range(10):
-        f = N.random.multivariate_normal(N.zeros(X.shape[0]),K)
-        P.plot(X[:,0],f,"b-")
+        f = np.random.multivariate_normal(np.zeros(X.shape[0]), K)
+        pl.plot(X[:, 0], f, "b-")
 
-    P.title(k)
-    P.axis('tight')
+    pl.title(k)
+    pl.axis('tight')
     count += 1
 
 if cfg.getboolean('examples', 'interactive', True):
     # show all the cool figures
-    P.show()
+    pl.show()
