@@ -26,7 +26,7 @@ mvpa2.atlases.base module contains support for various atlases
 import os.path as osp
 from mvpa2.base import externals
 
-if externals.exists('lxml', raise_=True):
+if externals.exists('lxml', raise_=True, exception=ImportError):
     from lxml import etree, objectify
 
 from mvpa2.base.dochelpers import enhanced_doc_string
@@ -144,8 +144,8 @@ class XMLBasedAtlas(BaseAtlas):
             self._load_images()
             if self._image is not None:
                 # Get extent and voxel dimensions, limiting to 3D
-                self._extent = np.abs(np.asanyarray(self._image.get_shape()[:3]))
-                self._voxdim = np.asanyarray(self._image.get_header().get_zooms()[:3])
+                self._extent = np.abs(np.asanyarray(self._image.shape[:3]))
+                self._voxdim = np.asanyarray(self._image.header.get_zooms()[:3])
                 self.relativeToOrigin = True
         # Assign transformation to get into voxel coordinates,
         # spaceT will be set accordingly
@@ -229,7 +229,7 @@ class XMLBasedAtlas(BaseAtlas):
         self._coordT = coordT           # lets store for debugging etc
         if self._image is not None:
             # Combine with the image's qform
-            coordT = Linear(np.linalg.inv(self._image.get_header().get_qform()),
+            coordT = Linear(np.linalg.inv(self._image.header.get_qform()),
                             previous=coordT)
         self._spaceT = SpaceTransformation(
             previous=coordT, to_real_space=False)
