@@ -12,8 +12,10 @@ __docformat__ = 'restructuredtext'
 
 import re
 import textwrap
+import warnings
 import numpy as np
 from mvpa2.base.state import IndexedCollectable
+from mvpa2.base.constraints import expand_contraint_spec
 
 if __debug__:
     from mvpa2.base import debug
@@ -22,6 +24,7 @@ _whitespace_re = re.compile('\n\s+|^\s+')
 
 __all__ = [ 'Parameter', 'KernelParameter' ]
 
+
 class Parameter(IndexedCollectable):
     """This class shall serve as a representation of a parameter.
 
@@ -40,23 +43,21 @@ class Parameter(IndexedCollectable):
     or we wouldn't know that it was changed
     Here is a list of possible additional attributes:
 
-    allowedtype : str
-      Description of what types are allowed
-    min
-      Minimum value
-    max
-      Maximum value
     step
       Increment/decrement step size hint for optimization
     """
 
-    def __init__(self, default, ro=False,  index=None,  value=None,
+    def __init__(self, default, constraints=None, ro=False,  index=None,  value=None,
                  name=None, doc=None, **kwargs):
         """Specify a Parameter with a default value and arbitrary
         number of additional attributes.
 
         Parameters
         ----------
+        constraints : callable
+          A functor that takes any input value, performs checks or type
+          conversions and finally returns a value that is appropriate for a
+          parameter or raises an exception.
         name : str
           Name of the parameter under which it should be available in its
           respective collection.
@@ -70,7 +71,32 @@ class Parameter(IndexedCollectable):
           cannot be changed
         value
           Actual value of the parameter to be assigned
+
+        Examples
+        --------
+        -ensure the parameter to be of type float
+        (None not allowed as value):
+        constraints = EnsureFloat()
+        >>> from mvpa2.base.param import Parameter
+        >>> from mvpa2.base.constraints import (EnsureFloat, EnsureRange,
+        ...                              AltConstraints, Constraints)
+        >>> C = Parameter(23.0, constraints=EnsureFloat())
+
+        -ensure the parameter to be of type float or to be None:
+        >>> C = Parameter(23.0, constraints=AltConstraints(EnsureFloat(), None))
+
+        -ensure the parameter to be None or to be of type float
+        and lie in the inclusive range (7.0,44.0):
+        >>> C = Parameter(23.0, AltConstraints(Constraints(EnsureFloat(),
+        ...                                    EnsureRange(min=7.0,max=44.0)),
+        ...                                    None))
         """
+        allowedtype = kwargs.pop('allowedtype', None)
+        if allowedtype is not None:
+            warnings.warn(
+                "allowedtype option was deprecated in favor of constraints. "
+                "Adjust your code, provided value '%s' was ignored"
+                % str(allowedtype), category=DeprecationWarning)
         # XXX probably is too generic...
         # and potentially dangerous...
         # let's at least keep track of what is passed
@@ -81,6 +107,7 @@ class Parameter(IndexedCollectable):
 
         self.__default = default
         self._ro = ro
+        self.constraints = expand_contraint_spec(constraints)
 
         # needs to come after kwargs processing, since some debug statements
         # rely on working repr()
@@ -106,13 +133,12 @@ class Parameter(IndexedCollectable):
         state = dict([(k, getattr(self, k)) for k in self._additional_props])
         state['_additional_props'] = self._additional_props
         state.update(icr[2])
-        res = (self.__class__, (self.__default, self._ro) + icr[1], state)
+        res = (self.__class__, (self.__default, self.constraints, self._ro) + icr[1], state)
         #if __debug__ and 'COL_RED' in debug.active:
         #    debug('COL_RED', 'Returning %s for %s' % (res, self))
         return res
 
 
-
     def __str__(self):
         res = IndexedCollectable.__str__(self)
         # it is enabled but no value is assigned yet
@@ -136,7 +162,7 @@ class Parameter(IndexedCollectable):
         if self._ro:
             s += ', ro=True'
         if not self.is_default:
-            s += ', value=%r' % self.value
+            s += ', value=%r' % (self.value, )
         s += ')'
         return s
 
@@ -149,14 +175,26 @@ class Parameter(IndexedCollectable):
         string or list of strings (if indent is None)
         """
         paramsdoc = '%s' % self.name
-        if hasattr(paramsdoc, 'allowedtype'):
-            paramsdoc += " : %s" % self.allowedtype
+        if not self.constraints is None:
+            sdoc = self.constraints.short_description()
+            if not sdoc is None:
+                if sdoc[0] == '(' and sdoc[-1] == ')':
+                    sdoc = sdoc[1:-1]
+                # parameters are always optional
+                paramsdoc += " : %s, optional" % sdoc
         paramsdoc = [paramsdoc]
+
         try:
             doc = self.__doc__.strip()
-            if not doc.endswith('.'): doc += '.'
+            if not doc.endswith('.'):
+                doc += '.'
+            if self.constraints is not None:
+                cdoc = self.constraints.long_description()
+                if cdoc[0] == '(' and cdoc[-1] == ')':
+                    cdoc = cdoc[1:-1]
+                doc += ' Constraints: %s.' % cdoc
             try:
-                doc += " (Default: %r)" % (self.default,)
+                doc += " [Default: %r]" % (self.default,)
             except:
                 pass
             # Explicitly deal with multiple spaces, for some reason
@@ -167,7 +205,6 @@ class Parameter(IndexedCollectable):
                                                  replace_whitespace=True)]
         except Exception, e:
             pass
-
         return '\n'.join(paramsdoc)
 
 
@@ -181,6 +218,11 @@ class Parameter(IndexedCollectable):
             self.value = self.__default
 
     def _set(self, val, init=False):
+        if self.constraints is not None:
+#            for c in self.constraints:
+#                val = c(val)
+#                #val = c.validate(val)
+            val = self.constraints(val)    
         different_value = self._value != val
         isarray = isinstance(different_value, np.ndarray)
         if self._ro and not init:
@@ -192,19 +234,6 @@ class Parameter(IndexedCollectable):
             if __debug__:
                 debug("COL",
                       "Parameter: setting %s to %s " % (str(self), val))
-            if not isarray:
-                if hasattr(self, 'min') and val < self.min:
-                    raise ValueError, \
-                          "Minimal value for parameter %s is %s. Got %s" % \
-                          (self.name, self.min, val)
-                if hasattr(self, 'max') and val > self.max:
-                    raise ValueError, \
-                          "Maximal value for parameter %s is %s. Got %s" % \
-                          (self.name, self.max, val)
-                if hasattr(self, 'choices') and (not val in self.choices):
-                    raise ValueError, \
-                          "Valid choices for parameter %s are %s. Got %s" % \
-                          (self.name, self.choices, val)
             self._value = val
             # Set 'isset' only if not called from initialization routine
             self._isset = not init #True
@@ -238,6 +267,7 @@ class Parameter(IndexedCollectable):
     default = property(fget=lambda x:x.__default, fset=_set_default)
     value = property(fget=lambda x:x._value, fset=_set)
 
+
 class KernelParameter(Parameter):
     """Just that it is different beast"""
     pass
