@@ -45,7 +45,7 @@ from textwrap import TextWrapper
 from mvpa2.misc.exceptions import UnknownStateError
 from mvpa2.base.attributes import IndexedCollectable, ConditionalAttribute
 from mvpa2.base.dochelpers import enhanced_doc_string, borrowdoc, _repr_attrs, \
-     get_docstring_split
+     get_docstring_split, _strid, _saferepr
 
 from mvpa2.base import externals
 # XXX local rename is due but later on
@@ -255,7 +255,7 @@ class Collection(BaseCollection):
           If True - do not complain about wrong key
         """
         if isinstance(key, basestring):
-            if key.upper() == 'ALL':
+            if key.lower() == 'all':
                 for key_ in self:
                     self._action(key_, func, missingok=missingok, **kwargs)
             else:
@@ -323,7 +323,7 @@ class ParameterCollection(Collection):
             # list only params with not default values
             if self[k].is_default:
                 continue
-            prefixes.append("%s=%r" % (k, self[k].value))
+            prefixes.append("%s=%s" % (k, _saferepr(self[k].value)))
         return prefixes
 
 
@@ -792,7 +792,7 @@ class ClassWithCollections(object):
 
         # need to check to avoid override of enabled ca in the case
         # of multiple inheritance, like both ClassWithCollectionsl and
-        # Harvestable
+        # Harvestable (note: Harvestable was refactored away)
         if '_collections' not in s__dict__:
             s__class__ = self.__class__
 
@@ -818,11 +818,28 @@ class ClassWithCollections(object):
         if __debug__:
             descr = kwargs.get('descr', None)
             debug("COL", "ClassWithCollections.__new__ was done "
-                  "for %s#%s with descr=%s",
-                  (s__class__.__name__, id(self), descr))
+                  "for %s%s with descr=%s",
+                  (s__class__.__name__, _strid(self), descr))
 
         return self
 
+    @classmethod
+    def _custom_kwargs_sort_items(cls, **kwargs):
+        """Custom sorting of kwargs to fulfill premises of pymvpa
+
+        Some time ago the world was square and we could assume
+        that enable_ca always comes before disable_ca.  But we
+        were misguided, thus now we need to provide a custom
+        sorting routine to sort disable_ca after enable_ca
+        """
+
+        def _key(x):
+            k = x[0]
+            if k == 'enable_ca':
+                return 'ZZZZZZ'
+            return k
+
+        return sorted(kwargs.items(), key=_key)
 
     def __init__(self, descr=None, **kwargs):
         """Initialize ClassWithCollections object
@@ -843,7 +860,8 @@ class ClassWithCollections(object):
             collections = self._collections
             # Assign attributes values if they are given among
             # **kwargs
-            for arg, argument in kwargs.items():
+
+            for arg, argument in self._custom_kwargs_sort_items(**kwargs):
                 isset = False
                 for collection in collections.itervalues():
                     if collection._is_initializable(arg):
@@ -886,8 +904,8 @@ class ClassWithCollections(object):
             #              + " Valid parameters are %s" % known_params
         if __debug__:
             debug("COL", "ClassWithCollections.__init__ was done "
-                  "for %s#%s with descr=%s",
-                  (self.__class__.__name__, id(self), descr))
+                  "for %s%s with descr=%s",
+                  (self.__class__.__name__, _strid(self), descr))
 
 
     #__doc__ = enhanced_doc_string('ClassWithCollections', locals())
@@ -996,13 +1014,17 @@ class ClassWithCollections(object):
         """
         prefixes = prefixes or []
         prefixes = prefixes[:]          # copy list
+        # filter using __init__doc__exclude__
+        for f in getattr(self, '__init__doc__exclude__', []):
+            prefixes = [x for x in prefixes if not x.startswith(f+'=')]
         id_str = ""
         module_str = ""
+
         if __debug__:
             if 'MODULE_IN_REPR' in debug.active:
                 fullname = True
             if 'ID_IN_REPR' in debug.active:
-                id_str = '#%r' % id(self)
+                id_str = _strid(self)
 
         if fullname:
             modulename = '%s' % self.__class__.__module__
