@@ -6,7 +6,7 @@
 #   copyright and license terms.
 #
 ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ##
-"""Classes for meta classifiers -- classifiers which use other classifiers
+"""Meta classifiers -- classifiers which use other classifiers or preprocessing
 
 Meta Classifiers can be grouped according to their function as
 
@@ -24,14 +24,16 @@ __docformat__ = 'restructuredtext'
 import numpy as np
 
 from mvpa2.misc.args import group_kwargs
-from mvpa2.base.types import is_sequence_type
+from mvpa2.base.types import is_sequence_type, asobjarray
 from mvpa2.base.param import Parameter
 
+from mvpa2.datasets import Dataset
+
 from mvpa2.generators.splitters import Splitter
 from mvpa2.generators.partition import NFoldPartitioner
 from mvpa2.datasets.miscfx import get_samples_by_attr
 from mvpa2.misc.attrmap import AttributeMap
-from mvpa2.base.dochelpers import _str
+from mvpa2.base.dochelpers import _str, _repr_attrs
 from mvpa2.base.state import ConditionalAttribute, ClassWithCollections
 
 from mvpa2.clfs.base import Classifier
@@ -55,7 +57,7 @@ if __debug__:
 class BoostedClassifier(Classifier):
     """Classifier containing the farm of other classifiers.
 
-    Should rarely be used directly. Use one of its childs instead
+    Should rarely be used directly. Use one of its children instead
     """
 
     # should not be needed if we have prediction_estimates upstairs
@@ -160,7 +162,9 @@ class BoostedClassifier(Classifier):
         was actually created. It will be used by
         MulticlassClassifier
         """
-        self.__clfs = clfs
+        # tuple to guarantee immutability since we are asssigning
+        # __tags__ below and rely on having clfs populated already
+        self.__clfs = tuple(clfs) if clfs is not None else tuple()
         """Classifiers to use"""
 
         if len(clfs):
@@ -347,6 +351,7 @@ class PredictionsCombiner(ClassWithCollections):
         dataset : Dataset
           training data in this case
         """
+        # TODO: implement stacking to help with resolving ties
         pass
 
 
@@ -437,8 +442,9 @@ class MaximalVote(PredictionsCombiner):
 
             if len(maxk) > 1:
                 warning("We got multiple labels %s which have the " % maxk +
-                        "same maximal vote %d. XXX disambiguate" % maxv)
-            predictions.append(maxk[0])
+                        "same maximal vote %d. XXX disambiguate. " % maxv +
+                        "Meanwhile selecting the first in sorted order")
+            predictions.append(sorted(maxk)[0])
 
         ca = self.ca
         ca.estimates = all_label_counts
@@ -458,7 +464,7 @@ class MeanPrediction(PredictionsCombiner):
         doc="Predictions from all classifiers are stored")
 
     def __call__(self, clfs, dataset):
-        """Actuall callable - perform meaning
+        """Actual callable - perform meaning
 
         """
         if len(clfs)==0:
@@ -534,25 +540,26 @@ class CombinedClassifier(BoostedClassifier):
     `PredictionsCombiner` functor.
     """
 
-    def __init__(self, clfs=None, combiner=None, **kwargs):
+    def __init__(self, clfs=None, combiner='auto', **kwargs):
         """Initialize the instance.
 
         Parameters
         ----------
         clfs : list of Classifier
           list of classifier instances to use
-        combiner : PredictionsCombiner
-          callable which takes care about combining multiple
-          results into a single one (e.g. maximal vote for
-          classification, MeanPrediction for regression))
+        combiner : PredictionsCombiner, optional
+          callable which takes care about combining multiple results into a single
+          one. If default ('auto') chooses `MaximalVote` for classification and
+          `MeanPrediction` for regression. If None is provided -- no combination is
+          done
         kwargs : dict
           dict of keyworded arguments which might get used
           by State or Classifier
 
-        NB: `combiner` might need to operate not on 'predictions' descrete
+        NB: `combiner` might need to operate not on 'predictions' discrete
             labels but rather on raw 'class' estimates classifiers
             estimate (which is pretty much what is stored under
-            `estimates`
+            `estimates`)
         """
         if clfs == None:
             clfs = []
@@ -560,7 +567,10 @@ class CombinedClassifier(BoostedClassifier):
         BoostedClassifier.__init__(self, clfs, **kwargs)
 
         self.__combiner = combiner
-        """Functor destined to combine results of multiple classifiers"""
+        """Input argument describing which "combiner" to use to combine results of multiple classifiers"""
+
+        self._combiner = None
+        """Actual combiner which would be decided upon later"""
 
 
     def __repr__(self, prefixes=[]):
@@ -573,12 +583,14 @@ class CombinedClassifier(BoostedClassifier):
     def combiner(self):
         # Decide either we are dealing with regressions
         # by looking at 1st learner
-        if self.__combiner is None:
-            self.__combiner = (
-                MaximalVote,
-                MeanPrediction)[int(self.clfs[0].__is_regression__)]()
-        return self.__combiner
-
+        if self._combiner is None:
+            if isinstance(self.__combiner, basestring) and self.__combiner == 'auto':
+                self._combiner = (
+                    MaximalVote,
+                    MeanPrediction)[int(self.clfs[0].__is_regression__)]()
+            else:
+                self._combiner = self.__combiner
+        return self._combiner
 
     def summary(self):
         """Provide summary for the `CombinedClassifier`.
@@ -596,9 +608,11 @@ class CombinedClassifier(BoostedClassifier):
         """Untrain `CombinedClassifier`
         """
         try:
-            self.__combiner.untrain()
+            self._combiner.untrain()
         except:
             pass
+        finally:
+            self._combiner = None
         super(CombinedClassifier, self)._untrain()
 
 
@@ -607,24 +621,30 @@ class CombinedClassifier(BoostedClassifier):
         """
         BoostedClassifier._train(self, dataset)
 
-        # combiner might need to train as well
-        self.combiner.train(self.clfs, dataset)
+        # combiner might need to be defined and trained as well at this point
+        if self.combiner is not None:
+            self.combiner.train(self.clfs, dataset)
 
 
     def _predict(self, dataset):
         """Predict using `CombinedClassifier`
         """
         ca = self.ca
-        cca = self.combiner.ca
-        BoostedClassifier._predict(self, dataset)
-        if ca.is_enabled("estimates"):
-            cca.enable('estimates')
-        # combiner will make use of conditional attributes instead of only predictions
-        # returned from _predict
-        predictions = self.combiner(self.clfs, dataset)
+        predictions = BoostedClassifier._predict(self, dataset)
+        if self.combiner is not None:
+            cca = self.combiner.ca
+            if ca.is_enabled("estimates"):
+                cca.enable('estimates')
+
+            # combiner will make use of conditional attributes instead of only predictions
+            # returned from _predict
+            predictions = self.combiner(self.clfs, dataset)
+        else:
+            cca = None
+
         ca.predictions = predictions
 
-        if ca.is_enabled("estimates"):
+        if ca.is_enabled("estimates") and cca is not None:
             if cca.is_active("estimates"):
                 # XXX or may be we could leave simply up to accessing .combiner?
                 ca.estimates = cca.estimates
@@ -637,7 +657,6 @@ class CombinedClassifier(BoostedClassifier):
         return predictions
 
 
-
 class TreeClassifier(ProxyClassifier):
     """`TreeClassifier` which allows to create hierarchy of classifiers
 
@@ -890,10 +909,14 @@ class TreeClassifier(ProxyClassifier):
 
         # now for predictions pointing to specific groups go into
         # corresponding one
-        # defer initialization since dtype would depend on predictions
-        predictions = None
+        predictions = np.zeros((len(dataset),),
+                               dtype=self.ca.trained_targets.dtype)
         for pred_group in set(clf_predictions):
-            gk = index2group[pred_group]
+            try:
+                gk = index2group[pred_group]
+            except IndexError:
+                raise IndexError("%s: Cannot obtain value for index %r among %r"
+                                 % (self, pred_group, index2group))
             clf_ = clfs[gk]
             group_indexes = (clf_predictions == pred_group)
             if __debug__:
@@ -901,14 +924,10 @@ class TreeClassifier(ProxyClassifier):
                       'Predicting for group %s using %s on %d samples',
                       (gk, clf_, np.sum(group_indexes)))
             if clf_ is None:
-                p = groups[gk][0] # our only label
+                predictions[group_indexes] = groups[gk][0] # our only label
             else:
-                p = clf_.predict(dataset[group_indexes])
-
-            if predictions is None:
-                predictions = np.zeros((len(dataset),),
-                                       dtype=np.asanyarray(p).dtype)
-            predictions[group_indexes] = p
+                predictions[group_indexes] = clf_.predict(dataset[group_indexes])
+                
         return predictions
 
 
@@ -933,22 +952,20 @@ class BinaryClassifier(ProxyClassifier):
         ProxyClassifier.__init__(self, clf, **kwargs)
 
         # Handle labels
-        sposlabels = set(poslabels)
-        sneglabels = set(neglabels)
 
         # TODO: move to use AttributeMap
         #self._attrmap = AttributeMap(dict([(l, -1) for l in sneglabels] +
         #                                  [(l, +1) for l in sposlabels]))
 
         # check if there is no overlap
-        overlap = sposlabels.intersection(sneglabels)
+        overlap = set(poslabels).intersection(neglabels)
         if len(overlap)>0:
             raise ValueError("Sets of positive and negative labels for " +
                 "BinaryClassifier must not overlap. Got overlap " %
                 overlap)
 
-        self.__poslabels = list(sposlabels)
-        self.__neglabels = list(sneglabels)
+        self.__poslabels = poslabels
+        self.__neglabels = neglabels
 
         # define what values will be returned by predict: if there is
         # a single label - return just it alone, otherwise - whole
@@ -1044,13 +1061,19 @@ class BinaryClassifier(ProxyClassifier):
 
 
 class MulticlassClassifier(CombinedClassifier):
-    """`CombinedClassifier` to perform multiclass using a list of
-    `BinaryClassifier`.
+    """Perform multiclass classification using a list of binary classifiers.
 
-    such as 1-vs-1 (ie in pairs like libsvm doesn) or 1-vs-all (which
-    is yet to think about)
+    Based on a `CombinedClassifier` for which it constructs a list of
+    binary 1-vs-1 (ie in pairs like LIBSVM does) or 1-vs-all (which is
+    yet to think about) classifiers.
     """
 
+    raw_predictions_ds = ConditionalAttribute(enabled=False,
+        doc="Wraps raw_predictions into a Dataset with .fa.(neg,pos) "
+        "describing actual labels used in each binary classification task "
+        "and samples containing actual decision labels per each input "
+        "sample")
+
     def __init__(self, clf, bclf_type="1-vs-1", **kwargs):
         """Initialize the instance
 
@@ -1080,9 +1103,9 @@ class MulticlassClassifier(CombinedClassifier):
         elif bclf_type == "1-vs-all": # TODO
             raise NotImplementedError
         else:
-            raise ValueError, \
-                  "Unknown type of classifier %s for " % bclf_type + \
-                  "BoostedMulticlassClassifier"
+            raise ValueError(
+                  "Unknown type of classifier %s for " % bclf_type +
+                  "MulticlassClassifier")
         self.__bclf_type = bclf_type
 
     # XXX fix it up a bit... it seems that MulticlassClassifier should
@@ -1092,36 +1115,71 @@ class MulticlassClassifier(CombinedClassifier):
                                             repr(self.__clf))
         return super(MulticlassClassifier, self).__repr__([prefix] + prefixes)
 
-
-    def _train(self, dataset):
-        """Train classifier
+    def _get_binary_pairs(self, dataset):
+        """Return a list of pairs of categories lists to be used in binary classification
         """
         targets_sa_name = self.get_space()
 
         # construct binary classifiers
         ulabels = dataset.sa[targets_sa_name].unique
+
         if self.__bclf_type == "1-vs-1":
             # generate pairs and corresponding classifiers
-            biclfs = []
-            for i in xrange(len(ulabels)):
-                for j in xrange(i+1, len(ulabels)):
-                    clf = self.__clf.clone()
-                    biclfs.append(
-                        BinaryClassifier(
-                            clf,
-                            poslabels=[ulabels[i]], neglabels=[ulabels[j]]))
+            # could use _product but let's stay inline with previuos
+            # implementation
+            label_pairs = [([ulabels[i]], [ulabels[j]])
+                           for i in xrange(len(ulabels))
+                           for j in xrange(i+1, len(ulabels))]
             if __debug__:
-                debug("CLFMC", "Created %d binary classifiers for %d labels",
-                      (len(biclfs), len(ulabels)))
-
-            self.clfs = biclfs
-
+                debug("CLFMC", "Created %d label pairs for original %d labels",
+                      (len(label_pairs), len(ulabels)))
         elif self.__bclf_type == "1-vs-all":
             raise NotImplementedError
 
+        return label_pairs
+
+    def _train(self, dataset):
+        """Train classifier
+        """
+        # construct binary classifiers
+        biclfs = []
+        for poslabels, neglabels in self._get_binary_pairs(dataset):
+            biclfs.append(
+                BinaryClassifier(self.__clf.clone(),
+                                 poslabels=poslabels,
+                                 neglabels=neglabels))
+        self.clfs = biclfs                # need to be set after, not operated in-place
         # perform actual training
         CombinedClassifier._train(self, dataset)
 
+    def _predict(self, dataset):
+        ca = self.ca
+        if ca.is_enabled("raw_predictions_ds"):
+            ca.enable("raw_predictions")
+
+        predictions = super(MulticlassClassifier, self)._predict(dataset)
+
+        if ca.is_enabled("raw_predictions_ds") or self.combiner is None:
+            if self.combiner is None:
+                raw_predictions = predictions
+            else:
+                # we should fetch those from ca
+                raw_predictions = ca.raw_predictions
+
+            # assign pos and neg to fa while squeezing out
+            # degenerate dimensions which are there to possibly accomodate
+            # 1-vs-all cases
+
+            # for consistency -- place into object array of tuples
+            # (Sensitivity analyzers already do the same)
+            pairs = zip(np.array([np.squeeze(clf.neglabels) for clf in self.clfs]).tolist(),
+                        np.array([np.squeeze(clf.poslabels) for clf in self.clfs]).tolist())
+            ca.raw_predictions_ds = raw_predictions_ds = \
+                Dataset(np.array(raw_predictions).T, fa={self.space: asobjarray(pairs)})
+        if self.combiner is None:
+            return raw_predictions_ds
+        else:
+            return predictions
 
 
 class SplitClassifier(CombinedClassifier):
@@ -1202,19 +1260,15 @@ class SplitClassifier(CombinedClassifier):
 
         clf_hastestdataset = hasattr(clf_template, 'testdataset')
 
-        # for proper and easier debugging - first define classifiers and then
-        # train them
-        for split in self.__partitioner.get_partition_specs(dataset):
+        self.ca.splits = []
+
+        for i, pset in enumerate(self.__partitioner.generate(dataset)):
             if __debug__:
                 debug("CLFSPL_", "Deepcopying %s for %s",
                       (clf_template, self))
             clf = clf_template.clone()
             bclfs.append(clf)
-        self.clfs = bclfs
 
-        self.ca.splits = []
-
-        for i, pset in enumerate(self.__partitioner.generate(dataset)):
             if __debug__:
                 debug("CLFSPL", "Training classifier for split %d", (i,))
 
@@ -1224,7 +1278,7 @@ class SplitClassifier(CombinedClassifier):
             if ca.is_enabled("splits"):
                 self.ca.splits.append(split)
 
-            clf = self.clfs[i]
+            clf = bclfs[i]
 
             # assign testing dataset if given classifier can digest it
             if clf_hastestdataset:
@@ -1253,6 +1307,9 @@ class SplitClassifier(CombinedClassifier):
             if ca.is_enabled("training_stats"):
                 # XXX this is broken, as it cannot deal with not yet set ca
                 ca.training_stats += clf.ca.training_stats
+        # need to be assigned after the entire list populated since
+        # _set_classifiers places them into a tuple
+        self.clfs = bclfs
 
 
     @group_kwargs(prefixes=['slave_'], passthrough=True)
@@ -1343,9 +1400,14 @@ class MappedClassifier(ProxyClassifier):
         return ProxyClassifier._predict(self, self.__mapper.forward(dataset))
 
 
-    def __str__(self):
-        return _str(self, '%s-%s' % (self.mapper, self.clf))
+    def __repr__(self, prefixes=[]):
+        return super(MappedClassifier, self).__repr__(
+            prefixes=prefixes
+            + _repr_attrs(self, ['mapper']))
 
+    def __str__(self, *args, **kwargs):
+        return super(MappedClassifier, self).__str__(
+              str(self.mapper), *args, **kwargs)
 
     mapper = property(lambda x:x.__mapper, doc="Used mapper")
 
