@@ -181,7 +181,18 @@ def _pvalue(x, cdf_func, rcdf_func, tail, return_tails=False, name=None):
     np.clip(pvalues, 0, 1.0, pvalues)
 
     # Assure that NaNs didn't get significant value
-    pvalues[np.isnan(x)] = 1.0
+    # TODO: should be moved into corresponding cdf/rcdf computation
+    #       since that is where x->pvalues relation can be assured
+    x_nans = np.isnan(x)
+    if np.any(x_nans):
+        if (isinstance(x, np.ndarray) and x.shape == pvalues.shape) \
+          or (pvalues.ndim == 1 and len(x) == len(pvalues)):
+            pvalues[x_nans] = 1.0
+        else:
+            raise ValueError(
+                "Input had NaN's but of different shape %s than output "
+                "pvalues %s, so cannot deduce what needs to be done. Please "
+                "make your input cleaner" % (x.shape, pvalues.shape))
 
     if is_scalar:
         pvalues = pvalues[0]
@@ -616,7 +627,7 @@ class AdaptiveNormal(AdaptiveNullDist):
 
 
 if externals.exists('scipy'):
-    from mvpa2.support.stats import scipy
+    from mvpa2.support.scipy.stats import scipy
     from scipy.stats import kstest
 
     """
@@ -1057,17 +1068,18 @@ if externals.exists('scipy'):
             if plot_cdf:
                 pl.plot(x, nonparam.cdf(x), 'k--', linewidth=1)
 
-            p_thr = p
-
             data_p = _pvalue(data, nonparam.cdf, nonparam.rcdf, tail)
-            data_p_thr = (data_p <= p_thr).ravel()
 
             npd = Nonparametric(data)
             x_p = _pvalue(x, npd.cdf, npd.rcdf, tail)
-            x_p_thr = np.abs(x_p) <= p_thr
-            # color bars which pass thresholding in red
-            for thr, bar_ in zip(x_p_thr[expand_tails:], hist[2]):
-                bar_.set_facecolor(('w','r')[int(thr)])
+
+            if p is not None:
+                data_p_thr = (data_p <= p).ravel()
+                x_p_thr = np.abs(x_p) <= p
+
+                # color bars which pass thresholding in red
+                for thr, bar_ in zip(x_p_thr[expand_tails:], hist[2]):
+                    bar_.set_facecolor(('w','r')[int(thr)])
 
             if not len(matches):
                 # no matches were provided
@@ -1085,12 +1097,13 @@ if externals.exists('scipy'):
                     label += '(D=%.2f)' % (D)
 
                 xcdf_p = np.abs(_pvalue(x, dist.cdf, rcdf, tail))
-                xcdf_p_thr = (xcdf_p <= p_thr).ravel()
+                if p is not None:
+                    xcdf_p_thr = (xcdf_p <= p).ravel()
 
                 if p is not None and legend > 2:
                     # We need to compare detection under given p
                     data_cdf_p = np.abs(_pvalue(data, dist.cdf, rcdf, tail))
-                    data_cdf_p_thr = (data_cdf_p <= p_thr).ravel()
+                    data_cdf_p_thr = (data_cdf_p <= p).ravel()
 
                     # true positives
                     tp = np.logical_and(data_cdf_p_thr, data_p_thr)
