@@ -29,6 +29,7 @@ from mvpa2.base.state import ConditionalAttribute, ClassWithCollections, \
      UnknownStateError
 from mvpa2.base.dochelpers import enhanced_doc_string, table2string
 from mvpa2.clfs.stats import auto_null_dist
+from mvpa2.support.due import due, Doi
 
 if __debug__:
     from mvpa2.base import debug
@@ -1229,7 +1230,7 @@ class BayesConfusionHypothesis(Node):
       [[0, 1], [2], [3, 4, 5]]
 
     This hypothesis represent the state where class 0 and 1 cannot be
-    distinguish from each other, but both 0 and 1 together can be distinguished
+    distinguished from each other, but both 0 and 1 together can be distinguished
     from class 2 and the group of 3, 4, and 5 -- where classes from the later
     group cannot be distinguished from one another.
 
@@ -1259,7 +1260,7 @@ class BayesConfusionHypothesis(Node):
           Name of the sample attribute in the output dataset where the
           hypothesis partition configurations will be stored.
         prior_Hs : array
-          Vector of priors for each hypotheses. Typically used in conjuction
+          Vector of priors for each hypotheses. Typically used in conjunction
           with an explicit set of possible hypotheses (see ``hypotheses``).
           If ``None`` a flat prior is assumed.
         log : bool
@@ -1282,6 +1283,10 @@ class BayesConfusionHypothesis(Node):
         self._postprob = postprob
         self._hypotheses = hypotheses
 
+    @due.dcite(
+        Doi("10.1016/j.patcog.2011.04.025"),
+        description="Bayesian hypothesis testing",
+        tags=["reference-implementation"])
     def _call(self, ds):
         from mvpa2.support.bayes.partitioner import Partition
         from mvpa2.support.bayes.partial_independence import compute_logp_H
