@@ -22,6 +22,8 @@ from mvpa2.clfs.gda import LDA, QDA
 from mvpa2.clfs.gnb import GNB
 from mvpa2.kernels.np import LinearKernel, SquaredExponentialKernel, \
      GeneralizedLinearKernel
+from mvpa2.featsel.rfe import SplitRFE
+from mvpa2.clfs.dummies import RandomClassifier
 
 # Helpers
 from mvpa2.base import externals, cfg
@@ -30,7 +32,7 @@ from mvpa2.mappers.fx import absolute_features, maxofabs_sample
 from mvpa2.clfs.smlr import SMLRWeights
 from mvpa2.featsel.helpers import FractionTailSelector, \
     FixedNElementTailSelector, RangeElementSelector
-
+from mvpa2.generators.partition import OddEvenPartitioner
 from mvpa2.featsel.base import SensitivityBasedFeatureSelection
 
 # Kernels
@@ -45,7 +47,11 @@ _KNOWN_INTERNALS = [ 'knn', 'binary', 'svm', 'linear',
         'libsvm', 'sg', 'meta', 'retrainable', 'gpr',
         'notrain2predict', 'ridge', 'blr', 'gnpp', 'enet', 'glmnet',
         'gnb', 'plr', 'rpy2', 'swig', 'skl', 'lda', 'qda',
-        'random-forest', 'extra-trees']
+        'random-forest', 'extra-trees', 'random',
+        # oneclass-binary can provide binary output for the labels
+        # oneclass would always output a single label but with
+        #  estimates/probabilities of interest
+        'oneclass', 'oneclass-binary']
 
 class Warehouse(object):
     """Class to keep known instantiated classifiers
@@ -75,6 +81,7 @@ class Warehouse(object):
         if matches is None:
             matches = {}
         self.__matches = matches
+        self.__descriptions = {}
 
     def __getitem__(self, *args):
         if isinstance(args[0], tuple):
@@ -127,6 +134,10 @@ class Warehouse(object):
             if not hasattr(item, '__tags__'):
                 raise ValueError, "Cannot register %s " % item + \
                       "which has no __tags__ defined"
+            if item.descr in self.__descriptions:
+                raise ValueError("Cannot register %s, " % item + \
+                      "an item with descriptions '%s' already exists" \
+                      % item.descr)
             if len(item.__tags__) == 0:
                 raise ValueError, "Cannot register %s " % item + \
                       "which has empty __tags__"
@@ -137,8 +148,13 @@ class Warehouse(object):
             else:
                 raise ValueError, 'Unknown clf internal(s) %s' % \
                       clf_internals.difference(self._known_tags)
+            # access by descr
+            self.__descriptions[item.descr] = item
         return self
 
+    def get_by_descr(self, descr):
+        return self.__descriptions[descr]
+
     @property
     def internals(self):
         """Known internal tags of the classifiers
@@ -150,12 +166,32 @@ class Warehouse(object):
         """
         return [(x.descr, x.__tags__) for x in self.__items]
 
+    def print_registered(self, *args):
+        if not len(args):
+            args = (slice(None))
+        import numpy as np
+        import textwrap
+        # sort by description
+        for lrn in sorted(self.__getitem__(args), key=lambda x: x.descr.lower()):
+            print '%s\n%s' % (
+                    lrn.descr,
+                    textwrap.fill(', '.join(np.unique(lrn.__tags__)), 70,
+                                  initial_indent=' ' * 4,
+                                  subsequent_indent=' ' * 4)
+                )
+
     @property
     def items(self):
         """Registered items
         """
         return self.__items
 
+    @property
+    def descriptions(self):
+        """Descriptions of registered items"""
+        return self.__descriptions.keys()
+
+
 clfswh = Warehouse(known_tags=_KNOWN_INTERNALS) # classifiers
 regrswh = Warehouse(known_tags=_KNOWN_INTERNALS) # regressions
 
@@ -176,6 +212,10 @@ clfswh += \
      [ MulticlassClassifier(SMLR(lm=0.1),
                             descr='Pairs+maxvote multiclass on SMLR(lm=0.1)') ]
 
+clfswh += [ RandomClassifier(descr="Random"),
+            RandomClassifier(same=True, descr="RandomSame"),
+            ]
+
 if externals.exists('libsvm'):
     from mvpa2.clfs.libsvmc import svm as libsvm
     clfswh._known_tags.update(libsvm.SVM._KNOWN_IMPLEMENTATIONS.keys())
@@ -327,6 +367,7 @@ if externals.exists('skl'):
         clfswh += SKLLearnerAdapter(sklRandomForestClassifier(),
                                      tags=['random-forest', 'linear', 'non-linear',
                                            'binary', 'multiclass',
+                                           'oneclass',
                                            'non-deterministic', 'needs_population',],
                                      descr='skl.RandomForestClassifier()')
 
@@ -342,6 +383,7 @@ if externals.exists('skl'):
         clfswh += SKLLearnerAdapter(sklExtraTreesClassifier(),
                                      tags=['extra-trees', 'linear', 'non-linear',
                                            'binary', 'multiclass',
+                                           'oneclass',
                                            'non-deterministic', 'needs_population',],
                                      descr='skl.ExtraTreesClassifier()')
 
@@ -354,7 +396,10 @@ if externals.exists('skl'):
 
 
     if _skl_version >= '0.8':
-        sklPLSRegression = _skl_import('pls', 'PLSRegression')
+        if _skl_version >= '0.14':
+            sklPLSRegression = _skl_import('cross_decomposition', 'PLSRegression')
+        else:
+            sklPLSRegression = _skl_import('pls', 'PLSRegression')
         # somewhat silly use of PLS, but oh well
         regrswh += SKLLearnerAdapter(sklPLSRegression(n_components=1),
                                      tags=['linear', 'regression'],
@@ -378,7 +423,9 @@ if externals.exists('skl'):
                                         descr='skl.LassoLars()')
 
         _elastic_net = SKLLearnerAdapter(
-            sklElasticNet(alpha=.01, rho=.3),
+            sklElasticNet(alpha=.01,
+                          **{'l1_ratio' if externals.versions['skl'] >= '0.13'
+                                        else 'rho': .3}),
             tags=['enet', 'regression', 'linear', # 'has_sensitivity',
                  'does_feature_selection'],
             descr='skl.ElasticNet()')
@@ -440,7 +487,6 @@ clfswh += \
            FractionTailSelector(0.05, mode='select', tail='upper')),
         descr="GNB on 5%(ANOVA)")
 
-
 # GPR
 if externals.exists('scipy'):
     from mvpa2.clfs.gpr import GPR
@@ -491,6 +537,16 @@ if len(clfswh['linear', 'svm']) > 0:
                 RangeElementSelector(mode='select')),
              descr="LinSVM on SMLR(lm=0.1) non-0")
 
+    _rfeclf = linearSVMC.clone()
+    clfswh += \
+         FeatureSelectionClassifier(
+             _rfeclf,
+             SplitRFE(
+                 _rfeclf,
+                 OddEvenPartitioner(),
+                 fselector=FractionTailSelector(
+                     0.2, mode='discard', tail='lower')),
+             descr="LinSVM with nested-CV RFE")
 
     clfswh += \
         FeatureSelectionClassifier(
@@ -608,7 +664,7 @@ if len(clfswh['linear', 'svm']) > 0:
     #   FeatureSelectionClassifier(
     #    clf = LinearCSVMC(),
     #    feature_selection = RFE(             # on features selected via RFE
-    #        sensitivity_analyzer=\
+    #        sensitivity_analyzer=
     #            rfesvm.get_sensitivity_analyzer(postproc=absolute_features()),
     #        transfer_error=TransferError(rfesvm),
     #        stopping_criterion=FixedErrorThresholdStopCrit(0.05),
@@ -625,7 +681,7 @@ if len(clfswh['linear', 'svm']) > 0:
     #   FeatureSelectionClassifier(
     #    clf = LinearCSVMC(),
     #    feature_selection = RFE(             # on features selected via RFE
-    #        sensitivity_analyzer=\
+    #        sensitivity_analyzer=
     #            rfesvm.get_sensitivity_analyzer(postproc=absolute_features()),
     #        transfer_error=TransferError(rfesvm),
     #        stopping_criterion=FixedErrorThresholdStopCrit(0.05),
