@@ -144,6 +144,7 @@ Notes
 __docformat__ = 'restructuredtext'
 
 from mvpa2.base import externals
+
 externals.exists('scipy', raise_=True)
 
 from scipy.io import loadmat, savemat, matlab
@@ -152,12 +153,13 @@ import numpy as np
 from mvpa2.datasets.base import Dataset
 from mvpa2.featsel.base import StaticFeatureSelection
 from mvpa2.mappers.base import ChainMapper
-from mvpa2.base.collections import Collection #ArrayCollectable
+from mvpa2.base.collections import Collection  # ArrayCollectable
 from mvpa2.misc.neighborhood import QueryEngineInterface
 from mvpa2.measures.searchlight import Searchlight
 from mvpa2.base import debug, warning
 
 
+
 def _numpy_array_astype_unsafe(arr, type_):
     '''
     Helper function to deal with API change in numpy.
@@ -172,6 +174,8 @@ def _numpy_array_astype_unsafe(arr, type_):
     else:
         return arr.astype(type_, casting='unsafe')
 
+
+
 def _from_singleton(x, ndim=2):
     '''
     If x is an array of shape (1,1,...1) with ndim dimensions it returns the
@@ -180,11 +184,10 @@ def _from_singleton(x, ndim=2):
     s = (1,) * ndim
     if x.shape != s:
         raise ValueError("Expected singleton shape %s for %s, found %s" %
-                                (s, x, x.shape,))
+                         (s, x, x.shape,))
     return x[(0,) * ndim]
 
 
-
 # dictionary indicating whether elements in .fa, .sa and .a must be
 # transposed for CoSMoMVPA data representation. As in CoSMoMVPA a dataset
 # with .samples PxQ the elements in .fa must be ?xQ, (only) .fa requires
@@ -192,6 +195,8 @@ def _from_singleton(x, ndim=2):
 _attr_fieldname2do_transpose = dict(fa=True, sa=False, a=False)
 _is_private_key = lambda s: s.startswith('__') and s.endswith('__')
 
+
+
 def _loadmat_internal(fn):
     '''
     Helper function to load matlab data
@@ -287,6 +292,7 @@ def _attributes_cosmo2dict(cosmo):
     return pymvpa_attributes
 
 
+
 def _attributes_dict2cosmo(ds):
     '''
     Converts attributes in Dataset to CoSMoMVPA-like attributes
@@ -323,15 +329,22 @@ def _attributes_dict2cosmo(ds):
             for k in attr_collection:
                 value = attr_collection[k].value
 
-                if len(value.shape) == 1:
-                    # transform vectors (shape=(P,)) to vectors in matrix
-                    # form (shape=(1,P))
-                    value = np.reshape(value, (-1, 1))
+                if isinstance(value, tuple):
+                    value = np.asarray(value)
 
-                if do_transpose:
-                    # feature attribute case, to match dimensionality
-                    # in second dimension
-                    value = value.T
+                try:
+                    if len(value.shape) == 1:
+                        # transform vectors (shape=(P,)) to vectors in matrix
+                        # form (shape=(1,P))
+                        value = np.reshape(value, (-1, 1))
+
+                    if do_transpose:
+                        # feature attribute case, to match dimensionality
+                        # in second dimension
+                        value = value.T
+                except AttributeError:
+                    # not supported data element, skip
+                    continue
 
                 dtypes.append((k, 'O'))
                 values.append(value)
@@ -372,6 +385,7 @@ def _mat_replace_matlab_function_by_string(x):
     return None
 
 
+
 def _mat_make_saveable(x, fixer=_mat_replace_matlab_function_by_string):
     '''
     Make a Matlab data structure saveable by scipy's matsave
@@ -411,7 +425,7 @@ def _mat_make_saveable(x, fixer=_mat_replace_matlab_function_by_string):
     if type(x) is dict:
         # use recursion
         return dict((k, _mat_make_saveable(v, fixer=fixer))
-                        for k, v in x.iteritems())
+                    for k, v in x.iteritems())
 
     elif isinstance(x, np.ndarray) and x.dtype.names is None:
         # standard array or object array
@@ -449,6 +463,7 @@ def _mat_make_saveable(x, fixer=_mat_replace_matlab_function_by_string):
         raise TypeError('Unexpected type %s in %s' % (type(x), x))
 
 
+
 def _check_cosmo_dataset(cosmo):
     '''
     Helper function to ensure a cosmo input for cosmo_dataset is valid.
@@ -476,11 +491,12 @@ def _check_cosmo_dataset(cosmo):
 
     if abs(decimals_nonzero) > warn_for_extreme_values_decimals:
         msg = ('Samples have extreme values, maximum absolute value is %s; '
-             'This may affect some analyses. Considering scaling the samples, '
-             'e.g. by a factor of 10**%d ' % (max_nonzero, -decimals_nonzero))
+               'This may affect some analyses. Considering scaling the samples, '
+               'e.g. by a factor of 10**%d ' % (max_nonzero, -decimals_nonzero))
         warning(msg)
 
 
+
 def cosmo_dataset(cosmo):
     '''
     Construct Dataset from CoSMoMVPA format
@@ -518,6 +534,7 @@ def cosmo_dataset(cosmo):
     return Dataset(**args)
 
 
+
 def map2cosmo(ds, filename=None):
     '''
     Convert PyMVPA Dataset to CoSMoMVPA struct saveable by scipy's savemat
@@ -555,6 +572,7 @@ def map2cosmo(ds, filename=None):
     return cosmo_fixed
 
 
+
 def from_any(x):
     '''
     Load CoSMoMVPA dataset or neighborhood
@@ -609,6 +627,7 @@ def from_any(x):
     raise ValueError('Unrecognized input %s' % x)
 
 
+
 class CosmoQueryEngine(QueryEngineInterface):
     '''
     queryengine for neighborhoods defined in CoSMoMVPA.
@@ -643,6 +662,8 @@ class CosmoQueryEngine(QueryEngineInterface):
     >> map2cosmo(ds_res,'result.mat')
 
     '''
+
+
     def __init__(self, mapping, a=None, fa=None):
         '''
         Parameters
@@ -688,6 +709,7 @@ class CosmoQueryEngine(QueryEngineInterface):
         attributes['samples'] = np.zeros((0, nfeatures), dtype=np.int_)
         self._dataset_template = Dataset(**attributes)[:, ids]
 
+
     @staticmethod
     def _check_mapping(mapping):
         '''
@@ -703,13 +725,13 @@ class CosmoQueryEngine(QueryEngineInterface):
                 raise ValueError('Keys %s must be int, found %s' % (k, type(k)))
             if not isinstance(v, np.ndarray):
                 raise TypeError('Value %s for key %s must be numpy array' %
-                                                                    (v, k))
+                                (v, k))
             if not np.issubdtype(np.int_, v.dtype):
                 raise ValueError('Value %s for key %s must be int' % (v, k))
 
 
     @classmethod
-    def from_mat(cls, neighbors, a=None, fa=None):
+    def from_mat(cls, neighbors, a=None, fa=None, origin=None):
         '''
         Create CosmoQueryEngine from mat struct
 
@@ -723,10 +745,15 @@ class CosmoQueryEngine(QueryEngineInterface):
             dataset attributes to be used for the output of a Searchlight
         fa: None or dict or ArrayCollectable
             dataset attributes to be used for the output of a Searchlight
+        origin:
+            Optional contents of .a and .fa of dataset indexed by neighbors;
+            its content is ignored
 
         Notes
         -----
-        Empty elements are ignored
+        Empty elements are ignored.
+        Future implementations may store the origin element, and check that
+        its contents agrees with a dataset when this instances trains on it.
         '''
 
         neighbors_vec = neighbors.ravel()
@@ -763,6 +790,7 @@ class CosmoQueryEngine(QueryEngineInterface):
                                                 template.a,
                                                 template.fa)
 
+
     def __str__(self):
         '''
         Return string summary of this instance
@@ -772,10 +800,11 @@ class CosmoQueryEngine(QueryEngineInterface):
         ids = self.ids
 
         return ('%s(%d center ids (%d .. %d), <fa: %s>, <a: %s>' %
-                        (self.__class__.__name__, len(ids),
-                         np.min(ids), np.max(ids),
-                         ', '.join(template.fa.keys()),
-                         ', '.join(template.a.keys())))
+                (self.__class__.__name__, len(ids),
+                 np.min(ids), np.max(ids),
+                 ', '.join(template.fa.keys()),
+                 ', '.join(template.a.keys())))
+
 
     def __reduce__(self):
         '''
@@ -784,27 +813,32 @@ class CosmoQueryEngine(QueryEngineInterface):
         template = self._dataset_template
         return (self.__class__, (self._mapping, template.a, template.fa))
 
+
     def __len__(self):
         '''
         Return number of ids (keys)
         '''
         return len(self.ids)
 
+
     def train(self, dataset):
         '''
         This method does nothing
         '''
         pass
 
+
     def untrain(self):
         '''
         This method does nothing
         '''
         pass
 
+
     def query(self, **kwargs):
         raise NotImplementedError
 
+
     def query_byid(self, id):
         '''
         Returns
@@ -816,6 +850,7 @@ class CosmoQueryEngine(QueryEngineInterface):
 
         return self._mapping[id]
 
+
     @property
     def ids(self):
         '''
@@ -827,6 +862,7 @@ class CosmoQueryEngine(QueryEngineInterface):
 
         return self._ids
 
+
     @property
     def a(self):
         '''
@@ -837,6 +873,7 @@ class CosmoQueryEngine(QueryEngineInterface):
         '''
         return self._dataset_template.a
 
+
     @property
     def fa(self):
         '''
@@ -848,6 +885,7 @@ class CosmoQueryEngine(QueryEngineInterface):
         '''
         return self._dataset_template.fa
 
+
     def set_output_dataset_attributes(self, ds):
         '''
         Set attributes to output dataset (e.g. after running a searchlight)
@@ -866,7 +904,7 @@ class CosmoQueryEngine(QueryEngineInterface):
         '''
         if not 'center_ids' in ds.fa:
             raise KeyError('Dataset seems not to be the result from '
-                            'running a searchlight: missing .fa.center_ids')
+                           'running a searchlight: missing .fa.center_ids')
 
         center_ids = ds.fa.center_ids
 
@@ -891,6 +929,7 @@ class CosmoQueryEngine(QueryEngineInterface):
         return ds_copy
 
 
+
 class CosmoSearchlight(Searchlight):
     '''
     Implement a standard Saerchlight measure, but with a separate
@@ -901,6 +940,7 @@ class CosmoSearchlight(Searchlight):
     either from a .mat matlab file or through a CosmoQueryEngine
     '''
 
+
     def __init__(self, datameasure, nbrhood, add_center_fa=False,
                  results_postproc_fx=None,
                  results_backend='native',
@@ -958,7 +998,7 @@ class CosmoSearchlight(Searchlight):
         expected_type = CosmoQueryEngine
         if not isinstance(queryengine, expected_type):
             raise TypeError('Queryengine should be a %s, found type %s' %
-                                    (expected_type, type(queryengine)))
+                            (expected_type, type(queryengine)))
 
         super(CosmoSearchlight, self).__init__(datameasure, queryengine,
                                                add_center_fa=add_center_fa,
@@ -969,6 +1009,7 @@ class CosmoSearchlight(Searchlight):
                                                nblocks=nblocks,
                                                **kwargs)
 
+
     def _call(self, ds):
         '''
         Perform standard searchlight analysis, then update
