@@ -28,6 +28,7 @@ from mvpa2.mappers.fx import maxofabs_sample, BinaryFxNode
 from mvpa2.base.dochelpers import _str
 from mvpa2.generators.base import Repeater
 
+from mvpa2.support.due import due, Doi, BibTeX
 
 import numpy as np
 from mvpa2.base.state import ConditionalAttribute
@@ -163,7 +164,24 @@ class RFE(IterativeFeatureSelection):
             prefixes=prefixes
             + _repr_attrs(self, ['update_sensitivity'], default=True))
 
-
+    @due.dcite(
+        BibTeX("""
+@Article{ GWB+02,
+    author = "I. Guyon and J. Weston and S. Barnhill and V. Vapnik",
+    title = "Gene Selection for Cancer Classification using Support Vector Machines",
+    volume = "46",
+    year = "2002",
+    pages = "389--422",
+    publisher = "Kluwer",
+    address = "Hingham, MA, USA",
+    journal = "Machine Learning"
+}"""),
+        description="Recursive feature elimination procedure",
+        tags=["implementation"])
+    @due.dcite(
+        Doi("10.1162/neco.2007.09-06-340"),
+        description="Full-brain fMRI decoding using SVM RFE",
+        tags=["use"])
     def _train(self, ds):
         """Proceed and select the features recursively eliminating less
         important ones.
@@ -174,7 +192,7 @@ class RFE(IterativeFeatureSelection):
           used to compute sensitivity maps and train a classifier
           to determine the transfer error
         testdataset : Dataset
-          used to test the trained classifer to determine the
+          used to test the trained classifier to determine the
           transfer error
 
         Returns a tuple of two new datasets with the feature subset of
