@@ -15,13 +15,15 @@ import inspect
 
 from mvpa2.base import warning
 from mvpa2.base.node import Node
+from mvpa2.base.param import Parameter
+from mvpa2.base.constraints import *
 from mvpa2.datasets import Dataset
 from mvpa2.base.dochelpers import _str, _repr_attrs
 from mvpa2.mappers.base import Mapper
 from mvpa2.misc.support import array_whereequal
 from mvpa2.base.dochelpers import borrowdoc
 
-from mvpa2.misc.transformers import sum_of_abs, max_of_abs
+from mvpa2.misc.transformers import sum_of_abs, max_of_abs, subtract_mean
 
 if __debug__:
     from mvpa2.base import debug
@@ -34,13 +36,14 @@ class FxMapper(Mapper):
     """Indicate that this mapper is always trained."""
 
     def __init__(self, axis, fx, fxargs=None, uattrs=None,
-                 attrfx='merge'):
+                 attrfx='merge', order='uattrs'):
         """
         Parameters
         ----------
         axis : {'samples', 'features'}
         fx : callable
         fxargs : tuple
+          Passed as *args to ``fx``
         uattrs : list
           List of attribute names to consider. All possible combinations
           of unique elements of these attributes are used to determine the
@@ -51,6 +54,16 @@ class FxMapper(Mapper):
           determined. If the content of the attribute is not uniform for a
           samples group a unique string representation is created.
           If `None`, attributes are not altered.
+        order : {'uattrs', 'occurrence', None}
+          If which order groups should be merged together.  If `None` (default
+          before 2.3.1), the order is imposed only by the order of
+          `uattrs` as keys in the dictionary, thus can vary from run to run.
+          If `'occurrence'`, groups will be ordered by the first occurrence
+          of group samples in original dataset. If `'uattrs'`, groups will be
+          sorted by the values of uattrs with follow-up attr having higher
+          importance for ordering (e .g. `uattrs=['targets', 'chunks']` would
+          order groups first by `chunks` and then by `targets` within each
+          chunk).
         """
         Mapper.__init__(self)
 
@@ -68,6 +81,8 @@ class FxMapper(Mapper):
             self.__attrfx = _uniquemerge2literal
         else:
             self.__attrfx = attrfx
+        assert(order in (None, 'uattrs', 'occurrence'))
+        self.__order = order
 
 
     @borrowdoc(Mapper)
@@ -77,8 +92,10 @@ class FxMapper(Mapper):
             + _repr_attrs(self, ['axis', 'fx', 'uattrs'])
             + _repr_attrs(self, ['fxargs'], default=())
             + _repr_attrs(self, ['attrfx'], default='merge')
+            + _repr_attrs(self, ['order'], default='uattrs')
             )
 
+
     def __str__(self):
         return _str(self, fx=self.__fx.__name__)
 
@@ -143,16 +160,8 @@ class FxMapper(Mapper):
     def _forward_dataset(self, ds):
         if self.__uattrs is None:
             mdata, sattrs = self._forward_dataset_full(ds)
-            single_attr = True
-            # yoh: Had another tentative solution but nope...  I guess
-            #      logic of wrapping into list should go into _full
-            #      and _grouped
-            #(len(mdata.shape) != len(ds.shape) \
-            #or
-            #(mdata.shape != ds.shape and mdata.shape[0] == 1))
         else:
             mdata, sattrs = self._forward_dataset_grouped(ds)
-            single_attr = False
 
         samples = np.atleast_2d(mdata)
 
@@ -167,24 +176,29 @@ class FxMapper(Mapper):
         if self.__axis == 'samples':
             out = ds.copy(deep=False, sa=[])
             col = out.sa
+            incol = ds.sa
             col.set_length_check(samples.shape[0])
         else:
             out = ds.copy(deep=False, fa=[])
             col = out.fa
+            incol = ds.fa
             col.set_length_check(samples.shape[1])
         # assign samples to do COW
         out.samples = samples
 
         for attr in sattrs:
             a = sattrs[attr]
-            # need to handle single literal attributes
-            if single_attr:
-                col[attr] = [a]
-            else:
-                # TODO -- here might puke if e.g it is a list where some items
-                # are empty lists... I guess just wrap in try/except and
-                # do dtype=object if catch
-                col[attr] = np.atleast_1d(a)
+            # TODO -- here might puke if e.g it is a list where some items
+            # are empty lists... I guess just wrap in try/except and
+            # do dtype=object if catch
+            a = np.atleast_1d(a)
+            # make sure we do not inflate the number of dimensions for no reason
+            # this could happen if there was only one unique value for an
+            # attribute and the default 'uniquemerge2literal' attrfx was given
+            if len(a.shape) > 1 and a.shape[-1] == 1 and attr in incol \
+                    and len(a.shape) > len(incol[attr].value.shape):
+                a.shape = a.shape[:-1]
+            col[attr] = a
 
         return out
 
@@ -207,10 +221,13 @@ class FxMapper(Mapper):
         self.__attrcombs = dict(zip(self.__uattrs,
                                 [col[attr].unique for attr in self.__uattrs]))
         # let it generate all combinations of unique elements in any attr
+        order = self.order
+        order_keys = []
         for comb in _orthogonal_permutations(self.__attrcombs):
             selector = reduce(np.multiply,
                                 [array_whereequal(col[attr].value, value)
                                  for attr, value in comb.iteritems()])
+
             # process the samples
             if axis == 0:
                 samples = ds.samples[selector]
@@ -228,10 +245,28 @@ class FxMapper(Mapper):
             mdata.append(fxed_samples)
             if not self.__attrfx is None:
                 # and now all samples attributes
-                fxed_attrs = [self.__attrfx(col[attr].value[selector])
-                                    for attr in col]
                 for i, attr in enumerate(col):
-                    attrs[attr].append(fxed_attrs[i])
+                    fxed_attr = self.__attrfx(col[attr].value[selector])
+                    attrs[attr].append(fxed_attr)
+            # possibly take care about collecting information to have groups ordered
+            if order == 'uattrs':
+                # reverse order as per docstring -- most of the time we have
+                # used uattrs=['targets', 'chunks'] and did expect chunks being
+                # groupped together.
+                order_keys.append([comb[a] for a in self.__uattrs[::-1]])
+            elif order == 'occurrence':
+                # First index should be sufficient since we are dealing
+                # with unique non-overlapping groups here (AFAIK ;) )
+                order_keys.append(np.where(selector)[0][0])
+
+        if order:
+            # reorder our groups using collected "order_keys"
+            # data
+            order_idxs = argsort(order_keys)
+            mdata = [mdata[i] for i in order_idxs]
+            # and attributes
+            attrs = dict((k, [v[i] for i in order_idxs])
+                         for k,v in attrs.iteritems())
 
         if axis == 0:
             mdata = np.vstack(mdata)
@@ -264,6 +299,7 @@ class FxMapper(Mapper):
     fxargs = property(fget=lambda self:self.__fxargs)
     uattrs = property(fget=lambda self:self.__uattrs)
     attrfx = property(fget=lambda self:self.__attrfx)
+    order = property(fget=lambda self:self.__order)
 
 #
 # Convenience functions to create some useful mapper with less complexity
@@ -287,13 +323,14 @@ def mean_sample(attrfx='merge'):
     return FxMapper('samples', np.mean, attrfx=attrfx)
 
 
-def mean_group_sample(attrs, attrfx='merge'):
+def mean_group_sample(attrs, attrfx='merge', **kwargs):
     """Returns a mapper that computes the mean samples of unique sample groups.
 
     The sample groups are identified by the unique combination of all
     values of a set of provided sample attributes.  Order of output
     samples might differ from original and correspond to sorted order
-    of corresponding `attrs`.
+    of corresponding `attrs`  by default.  Use `order='occurrence'` if you would
+    like to maintain the order.
 
     Parameters
     ----------
@@ -310,7 +347,7 @@ def mean_group_sample(attrs, attrfx='merge'):
     -------
     FxMapper instance.
     """
-    return FxMapper('samples', np.mean, uattrs=attrs, attrfx=attrfx)
+    return FxMapper('samples', np.mean, uattrs=attrs, attrfx=attrfx, **kwargs)
 
 
 def sum_sample(attrfx='merge'):
@@ -349,13 +386,21 @@ def mean_feature(attrfx='merge'):
     return FxMapper('features', np.mean, attrfx=attrfx)
 
 
-def mean_group_feature(attrs, attrfx='merge'):
+def subtract_mean_feature(attrfx='merge'):
+    """Subtract mean of features across samples.
+    Functionaly equivalent to MeanRemoval, but much slower.
+    """
+    return FxMapper('features', subtract_mean, attrfx=attrfx)
+
+
+def mean_group_feature(attrs, attrfx='merge', **kwargs):
     """Returns a mapper that computes the mean features of unique feature groups.
 
     The feature groups are identified by the unique combination of all values of
     a set of provided feature attributes.  Order of output
     features might differ from original and correspond to sorted order
-    of corresponding `attrs`.
+    of corresponding `attrs` by default.  Use `order='occurrence'` if you would
+    like to maintain the order.
 
     Parameters
     ----------
@@ -372,7 +417,7 @@ def mean_group_feature(attrs, attrfx='merge'):
     -------
     FxMapper instance.
     """
-    return FxMapper('features', np.mean, uattrs=attrs, attrfx=attrfx)
+    return FxMapper('features', np.mean, uattrs=attrs, attrfx=attrfx, **kwargs)
 
 
 def absolute_features():
@@ -396,6 +441,51 @@ def maxofabs_sample():
     """Returns a mapper that finds max of absolute values of all samples.
     """
     return FxMapper('samples', max_of_abs)
+
+
+class BinomialProportionCI(Mapper):
+    """Compute binomial proportion confidence intervals
+
+    This is a convenience frontend for binomial_proportion_ci_from_bool()
+    and supports all methods implemented in this function.
+
+    The confidence interval is computed independently for each feature column.
+    The returned dataset contains two samples.  The first one contains the
+    lower CI boundary and the second sample the upper boundary.
+
+    Returns
+    -------
+    dataset
+    """
+
+    is_trained = True
+
+    width = Parameter(.95,
+                      constraints=EnsureFloat() & EnsureRange(min=0, max=1),
+                      doc="Confidence interval width")
+    meth = Parameter('jeffreys',
+                     constraints=EnsureChoice('wald', 'wilson', 'agresti-coull',
+                                              'jeffreys', 'clopper-pearson',
+                                              'arc-sine', 'logit', 'anscombe'),
+                     doc="Interval estimation method")
+
+    def __init__(self, **kwargs):
+        Mapper.__init__(self, **kwargs)
+
+    def _train(self, ds):
+        pass
+
+    def _forward_data(self, data):
+        from mvpa2.misc.stats import binomial_proportion_ci_from_bool
+        return binomial_proportion_ci_from_bool(data, axis=0,
+                                                alpha=1-self.params.width,
+                                                meth=self.params.meth)
+
+    def _forward_dataset(self, ds):
+        msamp = self._forward_data(ds.samples)
+        mds = Dataset(msamp, sa=dict(ci_boundary=['lower', 'upper']))
+        return mds
+
 #
 # Utility functions
 #
@@ -412,23 +502,54 @@ def _uniquemerge2literal(attrs):
 
     Returns
     -------
-    Non-sequence arguments are passed as is. Sequences are converted into
-    a single item representation (see above) and returned.  None is returned
-    in case of an empty sequence.
+    Non-sequence arguments are passed as is, otherwise a sequences of unique
+    items is. None is returned in case of an empty sequence.
     """
     try:
-        unq = np.unique(attrs)
+        if isinstance(attrs[0], basestring):
+            # do not try to disassemble sequences of strings
+            raise TypeError
+        unq = [np.array(u) for u in set([tuple(p) for p in attrs])]
     except TypeError:
-        # so it is not an iterable -- return the original
-        return attrs
+        # either no 2d-iterable...
+        try:
+            unq = np.unique(attrs)
+        except TypeError:
+            # or no iterable at all -- return the original
+            return attrs
+
     lunq = len(unq)
     if lunq > 1:
-        return '+'.join([str(l) for l in unq])
-    elif lunq:                          # first entry (non
-        return unq[0]
+        return ['+'.join([str(l) for l in unq])]
+    elif lunq:
+        return unq
     else:
         return None
 
+def merge2first(attrs):
+    """Compress a sequence by discard all but the first element
+
+    This function can be useful as 'attrfx' argument for an FxMapper.
+
+    Parameters
+    ----------
+    attrs : sequence, arbitrary
+
+    Returns
+    -------
+    First element of the input sequence.
+    """
+    return attrs[0]
+
+def argsort(seq, reverse=False):
+    """Return indices to get sequence sorted
+    """
+    # Based on construct from
+    # http://stackoverflow.com/questions/3071415/efficient-method-to-calculate-the-rank-vector-of-a-list-in-python
+    # Thanks!
+
+    # cmp was not passed through since seems to be absent in python3
+    return sorted(range(len(seq)), key=seq.__getitem__, reverse=reverse)
 
 def _orthogonal_permutations(a_dict):
     """
@@ -515,3 +636,37 @@ class BinaryFxNode(Node):
         if np.isscalar(err):
             err = np.array(err, ndmin=2)
         return Dataset(err)
+
+
+class MeanRemoval(Mapper):
+    """Subtract sample mean from features."""
+
+    is_trained = True
+
+    in_place = Parameter(
+        False,
+        doc="""If False: a copy of the dataset will be made before demeaning.
+        If True: demeaning will be performed in-place, i.e. input data is
+        modified. This is faster, but can have side-effects when the original
+        dataset is used elsewhere again, and implies that floating point data
+        types are required to prevent rounding errors in this case.""",
+        constraints=EnsureBool())
+
+    def __init__(self, in_place=False, **kwargs):
+        Mapper.__init__(self, **kwargs)
+        self.in_place = in_place
+
+    def _forward_data(self, data):
+        mdata = data
+        mean = np.mean(mdata, axis=1)
+
+        if self.in_place:
+            if not np.issubdtype(mdata.dtype, float):
+                warning("Integer dtype. Mean removal won't work correctly for "
+                        "this implementation. Rounding errors will occur. "
+                        "Use in_place=False instead")
+            mdata -= mean[:, np.newaxis]
+
+        else:
+            mdata = mdata - mean[:, np.newaxis]
+        return mdata
