@@ -6,7 +6,7 @@
 #   copyright and license terms.
 #
 ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ##
-"""Mapper for data normalization by Z-Scoring."""
+"""Data normalization by Z-Scoring."""
 
 __docformat__ = 'restructuredtext'
 
@@ -132,6 +132,8 @@ class ZScoreMapper(Mapper):
                     params[c] = self._compute_params(ds.samples[slicer])
             else:
                 # global estimate
+                if isinstance(est_ids, set):
+                    est_ids = list(est_ids)
                 params = {'__all__': self._compute_params(ds.samples[est_ids])}
 
 
