@@ -6,8 +6,7 @@
 #   copyright and license terms.
 #
 ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ##
-"""An efficient implementation of searchlight for GNB.
-"""
+"""Support functionality for GNB and M1NN searchlights"""
 
 __docformat__ = 'restructuredtext'
 
@@ -124,8 +123,8 @@ class SimpleStatBaseSearchlight(BaseSearchlight):
     some stats are not needed (eg per sample X^2's) for M1NN, so we
     should make them optional depending on the derived class
 
-    Note
-    ----
+    Notes
+    -----
 
     refactored from the original GNBSearchlight
 
@@ -134,7 +133,7 @@ class SimpleStatBaseSearchlight(BaseSearchlight):
     # TODO: implement parallelization (see #67) and then uncomment
     __init__doc__exclude__ = ['nproc']
 
-    def __init__(self, generator, qe, errorfx=mean_mismatch_error,
+    def __init__(self, generator, queryengine, errorfx=mean_mismatch_error,
                  indexsum=None,
                  reuse_neighbors=False,
                  **kwargs):
@@ -144,8 +143,7 @@ class SimpleStatBaseSearchlight(BaseSearchlight):
         ----------
         generator : `Generator`
           Some `Generator` to prepare partitions for cross-validation.
-        qe : `QueryEngine`
-          Query engine which would provide neighborhood information
+          It must not change "targets", thus e.g. no AttributePermutator's
         errorfx : func, optional
           Functor that computes a scalar error value from the vectors of
           desired and predicted values (e.g. subclass of `ErrorFunction`).
@@ -161,7 +159,7 @@ class SimpleStatBaseSearchlight(BaseSearchlight):
         """
 
         # init base class first
-        BaseSearchlight.__init__(self, qe, **kwargs)
+        BaseSearchlight.__init__(self, queryengine, **kwargs)
 
         self._errorfx = errorfx
         self._generator = generator
@@ -324,6 +322,17 @@ class SimpleStatBaseSearchlight(BaseSearchlight):
         # The shape of results
         r_shape = (nrois,) + X.shape[2:]
 
+
+        def assign_ulabels(a):
+            out = np.empty(shape=a.shape, dtype=ulabels.dtype)
+            it = np.nditer([a, out],
+                    flags = ['external_loop', 'buffered'],
+                    op_flags = [['readonly'],
+                                ['writeonly', 'allocate', 'no_broadcast']])
+            for x, y in it:
+                y[...] = ulabels[x]
+            return it.operands[1]
+
         #
         # Everything toward optimization ;)
         #
@@ -349,9 +358,19 @@ class SimpleStatBaseSearchlight(BaseSearchlight):
         # XXX we could make it even more lightweight I guess...
         dataset_indicies = Dataset(np.arange(nsamples), sa=dataset.sa)
         splitter = Splitter(attr=generator.get_space())
-        splits = list(tuple(splitter.generate(ds_))
-                      for ds_ in generator.generate(dataset_indicies))
-        nsplits = len(splits)
+        partitions = list(generator.generate(dataset_indicies))
+        if __debug__:
+            for p in partitions:
+                if not (np.all(p.sa[targets_sa_name].value == labels)):
+                    raise NotImplementedError(
+                        "%s does not yet support partitioners altering the targets "
+                        "(e.g. permutators)" % self.__class__)
+
+        nsplits = len(partitions)
+        # ATM we need to keep the splits instead since they are used
+        # in two places in the code: step 2 and 5
+        splits = list(tuple(splitter.generate(ds_)) for ds_ in partitions)
+        del partitions                    # not used any longer
 
         # 2. Figure out the new 'chunks x labels' blocks of combinations
         #    of samples
@@ -399,7 +418,14 @@ class SimpleStatBaseSearchlight(BaseSearchlight):
         self._reserve_pl_stats_space((nlabels, ) + s_shape)
 
         # results
-        results = np.zeros((nsplits,) + r_shape)
+        if errorfx is mean_mismatch_error:
+            # if we know how it would look like, prepare the storage
+            results = np.zeros((nsplits,) + r_shape)
+        else:
+            # Otherwise delay assembling the results
+            results = []
+
+        all_targets, all_cvfolds = [], []
 
         # 4. Lets deduce all neighbors... might need to be RF into the
         #    parallel part later on
@@ -500,18 +526,40 @@ class SimpleStatBaseSearchlight(BaseSearchlight):
                 results[isplit, :] = \
                     (predictions != targets[:, None]).sum(axis=0) \
                     / float(len(targets))
-            else:
+                all_cvfolds += [isplit]
+            elif errorfx:
                 # somewhat silly but a way which allows to use pre-crafted
                 # error functions without a chance to screw up
-                for i, fpredictions in enumerate(predictions.T):
-                    results[isplit, i] = errorfx(fpredictions, targets)
+                result = np.atleast_2d(
+                    np.array([errorfx(fpredictions, targets)
+                              for fpredictions in predictions.T]))
+                results.append(result)
+                all_cvfolds += [isplit] * result.shape[0]
 
+            else:
+                # and if no errorfx -- we just need to assign original
+                # labels to the predictions BUT keep in mind that it is a matrix
+                results.append(assign_ulabels(predictions))
+                all_targets += [ulabels[i] for i in targets]
+                all_cvfolds += [isplit] * len(targets)
+
+            pass  # end of the split loop
+
+        if isinstance(results, list):
+            # we have just collected them, now they need to be vstacked
+            results = np.vstack(results)
+            assert(results.ndim >= 2)
 
         if __debug__:
             debug('SLC', "%s._call() is done in %.3g sec" %
                   (self.__class__.__name__, time.time() - time_start))
 
-        return Dataset(results)
+        out = Dataset(results)
+        if all_targets:
+            out.sa['targets'] = all_targets
+        out.sa['cvfolds'] = all_cvfolds
+        out.fa['center_ids'] = roi_ids
+        return out
 
     generator = property(fget=lambda self: self._generator)
     errorfx = property(fget=lambda self: self._errorfx)
