@@ -10,17 +10,18 @@
 
 __docformat__ = 'restructuredtext'
 
-import os
 import numpy as np
 
 from mvpa2.datasets.base import dataset_wizard, Dataset
-from mvpa2 import pymvpa_dataroot, pymvpa_datadbroot
 from mvpa2.misc.fx import get_random_rotation
+from mvpa2.base.dataset import vstack
+
+from mvpa2.misc.fx import double_gamma_hrf, single_gamma_hrf
+from mvpa2.misc.support import Event
 
 if __debug__:
     from mvpa2.base import debug
 
-##REF: Name was automagically refactored
 def multiple_chunks(func, n_chunks, *args, **kwargs):
     """Replicate datasets multiple times raising different chunks
 
@@ -32,22 +33,19 @@ def multiple_chunks(func, n_chunks, *args, **kwargs):
     -------
     ds : `mvpa2.datasets.base.Dataset`
     """
+    dss = []
     for chunk in xrange(n_chunks):
         ds_ = func(*args, **kwargs)
         # might not have chunks at all
-        if not ds_.sa.has_key('chunks'):
+        if not 'chunks' in ds_.sa:
             ds_.sa['chunks'] = np.repeat(chunk + 1, ds_.nsamples)
         else:
             ds_.sa.chunks[:] = chunk + 1
-        if chunk == 0:
-            ds = ds_
-        else:
-            ds.append(ds_)
+        dss.append(ds_)
 
-    return ds
+    return vstack(dss)
 
 
-##REF: Name was automagically refactored
 def dumb_feature_dataset():
     """Create a very simple dataset with 2 features and 3 labels
     """
@@ -60,7 +58,6 @@ def dumb_feature_dataset():
     return dataset_wizard(samples=np.array(data), targets=regs, chunks=range(len(regs)))
 
 
-##REF: Name was automagically refactored
 def dumb_feature_binary_dataset():
     """Very simple binary (2 labels) dataset
     """
@@ -73,10 +70,9 @@ def dumb_feature_binary_dataset():
     return dataset_wizard(samples=np.array(data), targets=regs, chunks=range(len(regs)))
 
 
-
 def normal_feature_dataset(perlabel=50, nlabels=2, nfeatures=4, nchunks=5,
-                         means=None, nonbogus_features=None, snr=3.0,
-                         normalize=True):
+                           means=None, nonbogus_features=None, snr=3.0,
+                           normalize=True):
     """Generate a univariate dataset with normal noise and specified means.
 
     Could be considered to be a generalization of
@@ -108,13 +104,13 @@ def normal_feature_dataset(perlabel=50, nlabels=2, nfeatures=4, nchunks=5,
       Divide by max(abs()) value to bring data into [-1, 1] range.
     """
 
-    data = np.random.standard_normal((perlabel*nlabels, nfeatures))
+    data = np.random.standard_normal((perlabel * nlabels, nfeatures))
     if snr != 0:
         data /= np.sqrt(snr)
     if (means is None) and (not nonbogus_features is None):
-        if len(nonbogus_features) > nlabels:
-            raise ValueError, "Can't assign simply a feature to a " + \
-                  "class: more nonbogus_features than labels"
+        if len(nonbogus_features) != nlabels:
+            raise ValueError(
+                "Provide as many nonbogus features as many labels you have")
         means = np.zeros((len(nonbogus_features), nfeatures))
         # pure multivariate -- single bit per feature
         for i in xrange(len(nonbogus_features)):
@@ -125,28 +121,29 @@ def normal_feature_dataset(perlabel=50, nlabels=2, nfeatures=4, nchunks=5,
     if normalize:
         # bring it 'under 1', since otherwise some classifiers have difficulties
         # during optimization
-        data = 1.0/(np.max(np.abs(data))) * data
+        data = 1.0 / (np.max(np.abs(data))) * data
     labels = np.concatenate([np.repeat('L%d' % i, perlabel)
-                                for i in range(nlabels)])
+                             for i in range(nlabels)])
     chunks = np.concatenate([np.repeat(range(nchunks),
-                                     perlabel//nchunks) for i in range(nlabels)])
+                                       perlabel // nchunks)
+                             for i in range(nlabels)])
     ds = dataset_wizard(data, targets=labels, chunks=chunks)
 
     # If nonbogus was provided -- assign .a and .fa accordingly
     if nonbogus_features is not None:
-        ds.fa['nonbogus_targets'] = np.array([None]*nfeatures)
+        ds.fa['nonbogus_targets'] = np.array([None] * nfeatures)
         ds.fa.nonbogus_targets[nonbogus_features] = ['L%d' % i for i in range(nlabels)]
         ds.a['nonbogus_features'] = nonbogus_features
         ds.a['bogus_features'] = [x for x in range(nfeatures)
                                   if not x in nonbogus_features]
-
-
     return ds
 
-##REF: Name was automagically refactored
-def pure_multivariate_signal(patterns, signal2noise = 1.5, chunks=None, targets=[0, 1]):
-    """ Create a 2d dataset with a clear multivariate signal, but no
-    univariate information.
+
+def pure_multivariate_signal(patterns, signal2noise=1.5, chunks=None,
+                             targets=None):
+    """ Create a 2d dataset with a clear purely multivariate signal.
+
+    This is known is the XOR problem.
 
     ::
 
@@ -155,19 +152,33 @@ def pure_multivariate_signal(patterns, signal2noise = 1.5, chunks=None, targets=
       %%%%%%%%%
       % X % O %
       %%%%%%%%%
+
+    Parameters
+    ----------
+    patterns: int
+      Number of data points in each of the four dot clouds
+    signal2noise: float, optional
+      Univariate signal pedestal.
+    chunks: array, optional
+      Vector for chunk labels for all generated samples.
+    targets: list, optional
+      Length-2 sequence of target values for both classes. If None,
+      [0, 1] is used.
     """
+    if targets is None:
+        targets = [0, 1]
 
     # start with noise
-    data = np.random.normal(size=(4*patterns, 2))
+    data = np.random.normal(size=(4 * patterns, 2))
 
     # add signal
-    data[:2*patterns, 1] += signal2noise
+    data[:2 * patterns, 1] += signal2noise
 
-    data[2*patterns:4*patterns, 1] -= signal2noise
+    data[2 * patterns:4 * patterns, 1] -= signal2noise
     data[:patterns, 0] -= signal2noise
-    data[2*patterns:3*patterns, 0] -= signal2noise
-    data[patterns:2*patterns, 0] += signal2noise
-    data[3*patterns:4*patterns, 0] += signal2noise
+    data[2 * patterns:3 * patterns, 0] -= signal2noise
+    data[patterns:2 * patterns, 0] += signal2noise
+    data[3 * patterns:4 * patterns, 0] += signal2noise
 
     # two conditions
     regs = np.array((targets[0:1] * patterns) + (targets[1:2] * 2 * patterns) + (targets[0:1] * patterns))
@@ -177,11 +188,9 @@ def pure_multivariate_signal(patterns, signal2noise = 1.5, chunks=None, targets=
     return dataset_wizard(samples=data, targets=regs, chunks=chunks)
 
 
-##REF: Name was automagically refactored
 def get_mv_pattern(s2n):
     """Simple multivariate dataset"""
-    return multiple_chunks(pure_multivariate_signal, 6,
-                          5, s2n, 1)
+    return multiple_chunks(pure_multivariate_signal, 6, 5, s2n, 1)
 
 
 def wr1996(size=200):
@@ -211,22 +220,21 @@ def wr1996(size=200):
     intervals = np.array([[-1.932, -0.453], [0.534, 3.142]])
     r = np.array([2.0, 1.3])
     x = np.random.rand(size, 2)
-    x *= np.array(intervals[:, 1]-intervals[:, 0])
+    x *= np.array(intervals[:, 1] - intervals[:, 0])
     x += np.array(intervals[:, 0])
     if __debug__:
         for i in xrange(2):
             debug('DG', '%d columnt Min: %g Max: %g' %
                   (i, x[:, i].min(), x[:, i].max()))
-    y = r[0]*np.cos(x[:, 0] + r[1]*np.cos(x.sum(1))) + \
-        np.random.randn(size)*np.sqrt(0.0025)
+    y = r[0] * np.cos(x[:, 0] + r[1] * np.cos(x.sum(1))) + \
+        np.random.randn(size) * np.sqrt(0.0025)
     y -= y.mean()
-    x34 = x + np.random.randn(size, 2)*0.02
+    x34 = x + np.random.randn(size, 2) * 0.02
     x56 = np.random.randn(size, 2)
     x = np.hstack([x, x34, x56])
     return dataset_wizard(samples=x, targets=y)
 
 
-##REF: Name was automagically refactored
 def sin_modulated(n_instances, n_features,
                   flat=False, noise=0.4):
     """ Generate a (quite) complex multidimensional non-linear dataset
@@ -235,17 +243,17 @@ def sin_modulated(n_instances, n_features,
     uniform noise
     """
     if flat:
-        data = (np.arange(0.0, 1.0, 1.0/n_instances)*np.pi)
+        data = (np.arange(0.0, 1.0, 1.0 / n_instances) * np.pi)
         data.resize(n_instances, n_features)
     else:
-        data = np.random.rand(n_instances, n_features)*np.pi
-    label = np.sin((data**2).sum(1)).round()
-    label += np.random.rand(label.size)*noise
+        data = np.random.rand(n_instances, n_features) * np.pi
+    label = np.sin((data ** 2).sum(1)).round()
+    label += np.random.rand(label.size) * noise
     return dataset_wizard(samples=data, targets=label)
 
-##REF: Name was automagically refactored
+
 def chirp_linear(n_instances, n_features=4, n_nonbogus_features=2,
-                data_noise=0.4, noise=0.1):
+                 data_noise=0.4, noise=0.1):
     """ Generates simple dataset for linear regressions
 
     Generates chirp signal, populates n_nonbogus_features out of
@@ -253,13 +261,13 @@ def chirp_linear(n_instances, n_features=4, n_nonbogus_features=2,
     signal itself with additional noise as labels
     """
     x = np.linspace(0, 1, n_instances)
-    y = np.sin((10 * np.pi * x **2))
+    y = np.sin((10 * np.pi * x ** 2))
 
-    data = np.random.normal(size=(n_instances, n_features ))*data_noise
+    data = np.random.normal(size=(n_instances, n_features)) * data_noise
     for i in xrange(n_nonbogus_features):
         data[:, i] += y[:]
 
-    labels = y + np.random.normal(size=(n_instances,))*noise
+    labels = y + np.random.normal(size=(n_instances,)) * noise
 
     return dataset_wizard(samples=data, targets=labels)
 
@@ -317,76 +325,12 @@ def linear1d_gaussian_noise(size=100, slope=0.5, intercept=1.0,
     """A straight line with some Gaussian noise.
     """
     x = np.linspace(start=x_min, stop=x_max, num=size)
-    noise = np.random.randn(size)*sigma
+    noise = np.random.randn(size) * sigma
     y = x * slope + intercept + noise
     return dataset_wizard(samples=x[:, None], targets=y)
 
 
-def load_example_fmri_dataset():
-    """Load minimal fMRI dataset that is shipped with PyMVPA."""
-    from mvpa2.datasets.mri import fmri_dataset
-    from mvpa2.misc.io import SampleAttributes
-
-    attr = SampleAttributes(os.path.join(pymvpa_dataroot, 'attributes.txt'))
-    ds = fmri_dataset(samples=os.path.join(pymvpa_dataroot, 'bold.nii.gz'),
-                      targets=attr.targets, chunks=attr.chunks,
-                      mask=os.path.join(pymvpa_dataroot, 'mask.nii.gz'))
-
-    return ds
-
-
-def load_datadb_tutorial_data(path=os.path.join(
-      pymvpa_datadbroot, 'tutorial_data', 'tutorial_data', 'data'),
-    roi='brain'):
-    """Loads the block-design demo dataset from PyMVPA dataset DB.
-
-    Parameters
-    ----------
-    path : str
-      Path of the directory containing the dataset files.
-    roi : str or int or tuple or None
-      Region Of Interest to be used for masking the dataset. If a string is
-      given a corresponding mask image from the demo dataset will be used
-      (mask_<str>.nii.gz). If an int value is given, the corresponding ROI
-      is determined from the atlas image (mask_hoc.nii.gz). If a tuple is
-      provided it may contain int values that a processed as explained
-      before, but the union of a ROIs is taken to produce the final mask.
-      If None, no masking is performed.
-    """
-    import nibabel as nb
-    from mvpa2.datasets.mri import fmri_dataset
-    from mvpa2.misc.io import SampleAttributes
-    if roi is None:
-        mask = None
-    elif isinstance(roi, str):
-        mask = os.path.join(path, 'mask_' + roi + '.nii.gz')
-    elif isinstance(roi, int):
-        nimg = nb.load(os.path.join(path, 'mask_hoc.nii.gz'))
-        tmpmask = nimg.get_data() == roi
-        mask = nb.Nifti1Image(tmpmask.astype(int), nimg.get_affine(),
-                              nimg.get_header())
-    elif isinstance(roi, tuple) or isinstance(roi, list):
-        nimg = nb.load(os.path.join(path, 'mask_hoc.nii.gz'))
-        tmpmask = np.zeros(nimg.get_shape(), dtype='bool')
-        for r in roi:
-            tmpmask = np.logical_or(tmpmask, nimg.get_data() == r)
-        mask = nb.Nifti1Image(tmpmask.astype(int), nimg.get_affine(),
-                              nimg.get_header())
-    else:
-        raise ValueError("Got something as mask that I cannot handle.")
-    attr = SampleAttributes(os.path.join(path, 'attributes.txt'))
-    ds = fmri_dataset(samples=os.path.join(path, 'bold.nii.gz'),
-                      targets=attr.targets, chunks=attr.chunks,
-                      mask=mask)
-    return ds
-
-
-load_datadb_demo_blockfmri = load_datadb_tutorial_data
-"""For backward compatibility with tutorial_lib which people might be
-   "using" already.  Deprecate entirely whenever tutorial_data gets updated.
-"""
-
-def autocorrelated_noise(ds, sr, cutoff, lfnl=3.0, bord=10, hfnl=None):
+def autocorrelated_noise(ds, sr, cutoff, lfnl=3.0, bord=10, hfnl=None, add_baseline=True):
     """Generate a dataset with samples being temporally autocorrelated noise.
 
     Parameters
@@ -432,7 +376,8 @@ def autocorrelated_noise(ds, sr, cutoff, lfnl=3.0, bord=10, hfnl=None):
     nsamples = lfilter(fb, fa, nsamples, axis=0)
 
     # add the pedestal
-    nsamples += msample
+    if add_baseline:
+        nsamples += msample
 
     # HF noise
     if not hfnl is None:
@@ -470,9 +415,107 @@ def random_affine_transformation(ds, scale_fac=100., shift_fac=10.):
     # reusing random data from dataset itself
     random_scale = samples[rndidx(len(ds)), rndidx(ds.nfeatures)] * scale_fac
     random_shift = samples[rndidx(len(ds))] * shift_fac
-    samples = np.dot(samples, R) * random_scale \
-              + random_shift
+    samples = np.dot(samples, R) * random_scale + random_shift
     return Dataset(samples, sa=ds.sa, fa=ds.fa,
                    a={'random_rotation': R,
                       'random_scale': random_scale,
                       'random_shift': random_shift})
+
+
+def simple_hrf_dataset(
+        events=[1, 20, 25, 50, 60, 90, 92, 140],
+        hrf_gen=lambda t: double_gamma_hrf(t) - single_gamma_hrf(t, 0.8, 1, 0.05),
+        fir_length=15,
+        nsamples=None,
+        tr=2.0,
+        tres=1,
+        baseline=800.0,
+        signal_level=1,
+        noise='normal',
+        noise_level=1,
+        resampling='scipy'):
+    """
+    events: list of Events or ndarray of onsets for simple(r) designs
+    """
+    if isinstance(events, np.ndarray) or not isinstance(events[0], dict):
+        events = [Event(onset=o) for o in events]
+    else:
+        assert(isinstance(events, list))
+        for e in events:
+            assert(isinstance(e, dict))
+
+    # play fmri
+    # full-blown HRF with initial dip and undershoot ;-)
+    hrf_x = np.arange(0, float(fir_length) * tres, tres)
+    if isinstance(hrf_gen, np.ndarray):
+        # just accept provided HRF and only verify size match
+        assert(len(hrf_x) == len(hrf_gen))
+        hrf = hrf_gen
+    else:
+        # actually generate it
+        hrf = hrf_gen(hrf_x)
+    if not nsamples:
+        # estimate number of samples needed if not provided
+        max_onset = max([e['onset'] for e in events])
+        nsamples = int(max_onset / tres + len(hrf_x) * 1.5)
+
+    # come up with an experimental design
+    fast_er = np.zeros(nsamples)
+    for e in events:
+        on = int(e['onset'] / float(tres))
+        off = int((e['onset'] + e.get('duration', 1.)) / float(tres))
+        if off == on:
+            off += 1                      # so we have at least 1 point
+        assert(range(on, off))
+        fast_er[on:off] = e.get('intensity', 1)
+    # high resolution model of the convolved regressor
+    model_hr = np.convolve(fast_er, hrf)[:nsamples]
+
+    # downsample the regressor to fMRI resolution
+    if resampling == 'scipy':
+        from scipy import signal
+        model_lr = signal.resample(model_hr,
+                                   int(tres * nsamples / tr),
+                                   window='ham')
+    elif resampling == 'naive':
+        if tr % tres != 0.0:
+            raise ValueError("You must use resample='scipy' since your TR=%.2g"
+                             " is not multiple of tres=%.2g" % (tr, tres))
+        if tr < tres:
+            raise ValueError("You must use resample='scipy' since your TR=%.2g"
+                             " is less than tres=%.2g" % (tr, tres))
+        step = int(tr // tres)
+        model_lr = model_hr[::step]
+    else:
+        raise ValueError("resampling can only be 'scipy' or 'naive'. Got %r"
+                         % resampling)
+
+    # generate artifical fMRI data: two voxels one is noise, one has
+    # something
+    wsignal = baseline + model_lr * signal_level
+    nsignal = np.ones(wsignal.shape) * baseline
+
+    # build design matrix: bold-regressor and constant
+    design = np.array([model_lr, np.repeat(1, len(model_lr))]).T
+
+    # two 'voxel' dataset
+    ds = dataset_wizard(samples=np.array((wsignal, nsignal)).T, targets=1)
+    ds.a['baseline'] = baseline
+    ds.a['tr'] = tr
+    ds.sa['design'] = design
+
+    ds.fa['signal_level'] = [signal_level, False]
+
+    if noise == 'autocorrelated':
+        # this one seems to be quite unstable and can provide really
+        # funky noise at times
+        noise = autocorrelated_noise(ds, 1 / tr, 1 / (2 * tr),
+                                     lfnl=noise_level, hfnl=noise_level,
+                                     add_baseline=False)
+    elif noise == 'normal':
+        noise = np.random.randn(*ds.shape) * noise_level
+    else:
+        raise ValueError(noise)
+    ds.sa['noise'] = noise
+    ds.samples += noise
+    return ds
