@@ -23,6 +23,39 @@ from mvpa2.misc.support import idhash as idhash_
 if __debug__:
     from mvpa2.base import debug
 
+class IdentityNeighborhood(object):
+    """Trivial neighborhood.
+
+    Use this if you want neighbors(i) == [i]
+    """
+
+    def __init__(self):
+        """Initialize the neighborhood"""
+        pass
+
+    def __repr__(self):
+        return self.__class__.__name__
+
+    def train(self, dataset):
+        pass
+
+    def __call__(self, coordinate):
+        """Return coordinate in a list
+
+        Parameters
+        ----------
+        coordinate : sequence type of length 3 with integers
+
+        Returns
+        -------
+        [tuple(coordinate)]
+
+        """
+        # type checking
+        coordinate = np.asanyarray(coordinate)
+        return [tuple(coordinate)]
+
+
 class Sphere(object):
     """N-Dimensional hypersphere.
 
@@ -232,7 +265,7 @@ class HollowSphere(Sphere):
 
     """
 
-    def __init__(self, radius, inner_radius, **kwargs):
+    def __init__(self, radius, inner_radius, include_center=False, **kwargs):
         """ Initialize the Sphere
 
         Parameters
@@ -245,6 +278,9 @@ class HollowSphere(Sphere):
           Inner radius of the 'sphere', describing where hollow
           part starts.  It is inclusive, so `inner_radius` of 0,
           would already remove the center element.
+        include_center : bool
+          Flag indicating whether to include the center element.
+          Center element is added as first feature. (Default: False)
         **kwargs
           See `Sphere` for additional keyword arguments
         """
@@ -253,10 +289,12 @@ class HollowSphere(Sphere):
                   "than the radius (got %g)" % (inner_radius, radius)
         Sphere.__init__(self, radius, **kwargs)
         self._inner_radius = inner_radius
+        self.include_center = include_center
 
     def __repr__(self, prefixes=[]):
         return super(HollowSphere, self).__repr__(
-            ['inner_radius=%r' % (self._inner_radius,)])
+            ['inner_radius=%r' % (self._inner_radius,)]
+            + _repr_attrs(self, ['include_center'], default=False))
 
     # Properties to assure R/O behavior for now
     @property
@@ -296,7 +334,7 @@ class HollowSphere(Sphere):
 
         if not len(res):
             warning("%s defines no neighbors" % self)
-        return res
+        return np.vstack([np.zeros(ndim,dtype='int'),res]) if self.include_center else res
 
 
 class QueryEngineInterface(object):
@@ -357,6 +395,7 @@ class QueryEngine(QueryEngineInterface):
         super(QueryEngine, self).__init__()
         self._queryobjs = kwargs
         self._queryattrs = {}
+        self._ids = None
 
 
     def __repr__(self, prefixes=[]):
@@ -364,6 +403,13 @@ class QueryEngine(QueryEngineInterface):
             prefixes=prefixes
             + ['%s=%r' % v for v in self._queryobjs.iteritems()])
 
+    def __len__(self):
+        return len(self._ids) if self._ids is not None else 0
+
+    @property
+    def ids(self):
+        return self._ids
+
     def train(self, dataset):
         # reset first
         self._queryattrs.clear()
@@ -372,6 +418,10 @@ class QueryEngine(QueryEngineInterface):
             self._queryattrs[space] = dataset.fa[space].value
         # execute subclass training
         self._train(dataset)
+        # by default all QueryEngines should with all features of the dataset
+        # Situation might be different in case of e.g. Surface-based
+        # searchlights.
+        self._ids = range(dataset.nfeatures)
 
 
     def query_byid(self, fid):
@@ -585,7 +635,7 @@ class CachedQueryEngine(QueryEngineInterface):
           If `dataset`'s .fa were changed -- it would raise an
           exception telling to `untrain` explicitly, since the idea is
           to reuse CachedQueryEngine with the same engine and same
-          dataset (up to variation of .sa, such as labels permutation
+          dataset (up to variation of .sa, such as labels permutation)
         """
         ds_fa_hash = idhash_(dataset.fa) + ':%d' % dataset.fa._uniform_length
         if self._trained_ds_fa_hash is None:
@@ -677,7 +727,7 @@ def scatter_neighborhoods(neighbor_gen, coords, deterministic=False):
       Callable that return a list of neighborhood element coordinates, when
       called with a seed coordinate (cf. Sphere)
     coords : list
-      List of candidate corrdinates that can serve as neighborhood seeds or
+      List of candidate coordinates that can serve as neighborhood seeds or
       elements.
     deterministic : bool
       If true, performs seed placement using an OrderedDict (available in
