@@ -14,8 +14,10 @@ if __debug__:
     from mvpa2.base import debug
     debug('INIT', 'mvpa2.misc.plot start')
 
-from mvpa2.misc.plot.base import *
+from mvpa2.base import externals
+
+if externals.exists('matplotlib'):
+    from mvpa2.misc.plot.base import *
 
 if __debug__:
     debug('INIT', 'mvpa2.misc.plot end')
-
