@@ -59,6 +59,8 @@ else:
 
 __sdebug('algorithms')
 from mvpa2.algorithms.hyperalignment import *
+if externals.exists('scipy') :
+    from mvpa2.algorithms.group_clusterthr import *
 
 __sdebug('clfs')
 from mvpa2 import clfs
@@ -119,7 +121,10 @@ from mvpa2.datasets.eep import *
 from mvpa2.datasets.eventrelated import *
 if externals.exists('nibabel') :
     from mvpa2.datasets.mri import *
+    from mvpa2.datasets.gifti import map2gifti, gifti_dataset
 from mvpa2.datasets.sources import *
+from mvpa2.datasets.sources.native import *
+from mvpa2.datasets.sources.openfmri import *
 from mvpa2.datasets import niml
 from mvpa2.datasets.niml import from_niml, to_niml
 from mvpa2.datasets import eeglab
@@ -169,6 +174,7 @@ if externals.exists('mdp'):
 if externals.exists('mdp ge 2.4'):
     from mvpa2.mappers.lle import *
 from mvpa2.mappers.glm import *
+from mvpa2.mappers.skl_adaptor import *
 
 __sdebug('measures')
 from mvpa2 import measures
@@ -248,7 +254,8 @@ if externals.exists("lxml") and externals.exists("nibabel"):
 __sdebug("surface searchlight")
 from mvpa2.misc.surfing.queryengine import SurfaceVerticesQueryEngine, \
                                            SurfaceVoxelsQueryEngine, \
-                                            disc_surface_queryengine
+                                           SurfaceQueryEngine, \
+                                           disc_surface_queryengine
 
 from mvpa2.misc.surfing import surf_voxel_selection, volgeom, \
                                 volsurf, volume_mask_dict
