@@ -6,9 +6,15 @@ data behaves as a standard normal under H_0.
 
 Author : Bertrand Thirion, 2008-2009
 """
+# For scipy import
+from __future__ import absolute_import
+
 import numpy as np
 from numpy.linalg import pinv
-import scipy.stats as st
+
+from mvpa2.base import externals
+if externals.exists('scipy', raise_=True):
+    import scipy.stats as st
 
 class FDR(object):
     """
