@@ -11,17 +11,20 @@
 import os
 import unittest
 
-from mvpa2.testing.tools import *
 from mvpa2.base.info import wtf
+from mvpa2.testing.tools import *
 
 @with_tempfile()
 def test_wtf(filename):
-    """Very basic testing -- just to see if it doesn't crash"""
+    """Very basic testing of wtf()"""
 
     sinfo = str(wtf())
-    sinfo_excludes = str(wtf(exclude=['process']))
-    ok_(len(sinfo) > len(sinfo_excludes))
-    ok_(not 'Process Info' in sinfo_excludes)
+    sinfo_excludes = str(wtf(exclude=['runtime']))
+    ok_(len(sinfo) > len(sinfo_excludes),
+        msg="Got not less info when excluded runtime."
+        " Original one was:\n%s and without process:\n%s"
+        % (sinfo, sinfo_excludes))
+    ok_(not 'RUNTIME' in sinfo_excludes)
 
     # check if we could store and load it back
     wtf(filename)
