@@ -36,10 +36,11 @@ class CmdlineHelpersTest(unittest.TestCase):
         v = split_comma_semicolon_lists(s, dtype=dtype)
         assert_equal(v, t)
 
-def suite():
+def suite():  # pragma: no cover
     return unittest.makeSuite(CmdlineHelpersTest)
 
 
-if __name__ == '__main__':
+if __name__ == '__main__':  # pragma: no cover
     import runner
+    runner.run()
 
