@@ -26,10 +26,11 @@ class ConfigTests(unittest.TestCase):
 
 
 
-def suite():
+def suite():  # pragma: no cover
     return unittest.makeSuite(ConfigTests)
 
 
-if __name__ == '__main__':
+if __name__ == '__main__':  # pragma: no cover
     import runner
+    runner.run()
 
