@@ -19,7 +19,6 @@ from mvpa2.algorithms.hyperalignment import Hyperalignment
 from mvpa2.mappers.zscore import zscore
 from mvpa2.misc.support import idhash
 from mvpa2.misc.data_generators import random_affine_transformation
-from mvpa2.misc.fx import get_random_rotation
 
 # Somewhat slow but provides all needed ;)
 from mvpa2.testing import sweepargs, reseed_rng
@@ -30,8 +29,8 @@ from mvpa2.generators.partition import NFoldPartitioner
 # if you need some classifiers
 #from mvpa2.testing.clfs import *
 
-class HyperAlignmentTests(unittest.TestCase):
 
+class HyperAlignmentTests(unittest.TestCase):
 
     @sweepargs(zscore_all=(False, True))
     @sweepargs(zscore_common=(False, True))
@@ -49,7 +48,7 @@ class HyperAlignmentTests(unittest.TestCase):
         # lets select for now only meaningful features
         ds_orig = ds4l[:, ds4l.a.nonbogus_features]
         nf = ds_orig.nfeatures
-        n = 4 # # of datasets to generate
+        n = 4  # # of datasets to generate
         Rs, dss_rotated, dss_rotated_clean, random_shifts, random_scales \
             = [], [], [], [], []
 
@@ -91,12 +90,14 @@ class HyperAlignmentTests(unittest.TestCase):
 
             idhashes_ = [idhash(ds.samples) for ds in dss]
             idhashes_targets_ = [idhash(ds.targets) for ds in dss]
-            self.assertEqual(idhashes, idhashes_,
+            self.assertEqual(
+                idhashes, idhashes_,
                 msg="Hyperalignment must not change original data.")
-            self.assertEqual(idhashes_targets, idhashes_targets_,
+            self.assertEqual(
+                idhashes_targets, idhashes_targets_,
                 msg="Hyperalignment must not change original data targets.")
 
-            self.assertEqual(ref_ds, ha.ca.choosen_ref_ds)
+            self.assertEqual(ref_ds, ha.ca.chosen_ref_ds)
 
             # Map data back
 
@@ -107,8 +108,8 @@ class HyperAlignmentTests(unittest.TestCase):
             nddss = []
             ndcss = []
             ds_orig_Rref = np.dot(ds_orig.samples, Rs[ref_ds]) \
-                           * random_scales[ref_ds] \
-                           + random_shifts[ref_ds]
+                * random_scales[ref_ds] \
+                + random_shifts[ref_ds]
             if zscore_common or zscore_all:
                 zscore(Dataset(ds_orig_Rref), chunks_attr=None)
             for ds_back in dss_clean_back:
@@ -125,32 +126,34 @@ class HyperAlignmentTests(unittest.TestCase):
             do_labile = cfg.getboolean('tests', 'labile', default='yes')
             if not noisy or do_labile:
                 # First compare correlations
-                self.assertTrue(np.all(np.array(ndcss)
-                                       >= (0.9, 0.85)[int(noisy)]),
-                        msg="Should have reconstructed original dataset more or"
-                        " less. Got correlations %s in %s case."
-                        % (ndcss, snoisy))
+                self.assertTrue(
+                    np.all(np.array(ndcss) >= (0.9, 0.85)[int(noisy)]),
+                    msg="Should have reconstructed original dataset more or"
+                    " less. Got correlations %s in %s case."
+                    % (ndcss, snoisy))
                 if not (zscore_all or zscore_common):
                     # if we didn't zscore -- all of them should be really close
-                    self.assertTrue(np.all(np.array(nddss)
-                                       <= (1e-10, 1e-1)[int(noisy)]),
+                    self.assertTrue(
+                        np.all(np.array(nddss) <= (1e-10, 1e-1)[int(noisy)]),
                         msg="Should have reconstructed original dataset well "
                         "without zscoring. Got normed differences %s in %s case."
                         % (nddss, snoisy))
                 elif do_labile:
                     # otherwise they all should be somewhat close
-                    self.assertTrue(np.all(np.array(nddss)
-                                           <= (.2, 3)[int(noisy)]),
+                    self.assertTrue(
+                        np.all(np.array(nddss) <= (.2, 3)[int(noisy)]),
                         msg="Should have reconstructed original dataset more or"
                         " less for all. Got normed differences %s in %s case."
                         % (nddss, snoisy))
-                    self.assertTrue(np.all(nddss[ref_ds] <= .09),
+                    self.assertTrue(
+                        np.all(nddss[ref_ds] <= .09),
                         msg="Should have reconstructed original dataset quite "
                         "well even with zscoring. Got normed differences %s "
                         "in %s case." % (nddss, snoisy))
                     # yoh: and leave 5% of difference for a chance and numerical
                     #      fluctuations ;)
-                    self.assertTrue(np.all(np.array(nddss) >= 0.95*nddss[ref_ds]),
+                    self.assertTrue(
+                        np.all(np.array(nddss) >= 0.95 * nddss[ref_ds]),
                         msg="Should have reconstructed orig_ds best of all. "
                         "Got normed differences %s in %s case with ref_ds=%d."
                         % (nddss, snoisy, ref_ds))
@@ -160,8 +163,8 @@ class HyperAlignmentTests(unittest.TestCase):
                             enable_ca=['training_residual_errors',
                                        'residual_errors'])
         mappers = ha(dss_rotated_clean)
-        self.assertTrue(np.all(ha.ca.training_residual_errors.sa.levels ==
-                              ['1', '2:0', '2:1']))
+        self.assertTrue(
+            np.all(ha.ca.training_residual_errors.sa.levels == ['1', '2:0', '2:1']))
         rterrors = ha.ca.training_residual_errors.samples
         # just basic tests:
         self.assertEqual(rterrors[0, ref_ds], 0)
@@ -169,8 +172,42 @@ class HyperAlignmentTests(unittest.TestCase):
         rerrors = ha.ca.residual_errors.samples
         self.assertEqual(rerrors.shape, (1, n))
 
+    def test_hypal_michael_caused_problem(self):
+        from mvpa2.misc import data_generators
+        from mvpa2.mappers.zscore import zscore
+        # Fake data
+        ds = data_generators.normal_feature_dataset(nfeatures=20)
+        ds_all = [data_generators.random_affine_transformation(ds) for i in range(3)]
+        _ = [zscore(sd, chunks_attr=None) for sd in ds_all]
+        # Making random data per subject for testing with bias added to first subject
+        ds_test = [np.random.rand(1, ds.nfeatures) for i in range(len(ds_all))]
+        ds_test[0] += np.arange(1, ds.nfeatures + 1) * 100
+        assert(np.corrcoef(ds_test[2], ds_test[1])[0, 1] < 0.99)  # that would have been rudiculous if it was
 
-    def _test_on_swaroop_data(self):
+        # Test with varying alpha so we for sure to not have that issue now
+        for alpha in (0, 0.01, 0.5, 0.99, 1.0):
+            hyper09 = Hyperalignment(alpha=alpha)
+            mappers = hyper09([sd for sd in ds_all])
+            ds_test_a = [m.forward(sd) for m, sd in zip(mappers, ds_test)]
+            ds_test_a = [mappers[0].reverse(sd) for sd in ds_test_a]
+            corr = np.corrcoef(ds_test_a[2], ds_test_a[1])[0, 1]
+            assert(corr < 0.99)
+
+    def test_hyper_ref_ds_range_checks(self):
+        # If supplied ref_ds can't be fit into non-negative int
+        # it should thrown an exception
+        with self.assertRaises(ValueError):
+            ha = Hyperalignment(ref_ds=-1.5)
+        # But work if it can fit, int(-0.5)=0
+        ha = Hyperalignment(ref_ds=0.5)
+        # or int(3.5)=3
+        ha = Hyperalignment(ref_ds=3.5)
+        # if ref_ds is out of range...
+        ds_all = [datasets['uni4small'] for i in range(3)]
+        # Making sure it raises error if ref_ds is out of range
+        self.assertRaises(ValueError, ha, ds_all)
+
+    def _test_on_swaroop_data(self):  # pragma: no cover
         #
         print "Running swaroops test on data we don't have"
         #from mvpa2.datasets.miscfx import zscore
@@ -180,8 +217,9 @@ class HyperAlignmentTests(unittest.TestCase):
         subj = ['cb', 'dm', 'hj', 'kd', 'kl', 'mh', 'ph', 'rb', 'se', 'sm']
         ds = []
         for sub in subj:
-            ds.append(fmri_dataset(samples=sub+'_movie.nii.gz',
-                                   mask=sub+'_mask_vt.nii.gz'))
+            ds.append(
+                fmri_dataset(samples=sub + '_movie.nii.gz',
+                             mask=sub + '_mask_vt.nii.gz'))
 
         '''
         Compute feature ranks in each dataset
@@ -277,11 +315,10 @@ class HyperAlignmentTests(unittest.TestCase):
         pass
 
 
-
-def suite():
+def suite():  # pragma: no cover
     return unittest.makeSuite(HyperAlignmentTests)
 
 
-if __name__ == '__main__':
+if __name__ == '__main__':  # pragma: no cover
     import runner
-
+    runner.run()
