@@ -220,7 +220,8 @@ class ErrorsTests(unittest.TestCase):
         train = datasets['uni2medium']
 
         num_perm = 10
-        permutator = AttributePermutator('targets', count=num_perm)
+        permutator = AttributePermutator('targets', count=num_perm,
+                                         limit='chunks')
         # define class to estimate NULL distribution of errors
         # use left tail of the distribution since we use MeanMatchFx as error
         # function and lower is better
@@ -305,7 +306,11 @@ class ErrorsTests(unittest.TestCase):
             stats = cv.ca.stats.stats
             Nlabels = len(ds.uniquetargets)
             # so we at least do slightly above chance
-            self.assertTrue(stats['ACC'] > 1.2 / Nlabels)
+            # But LARS manages to screw up there as well ATM from time to time, so making
+            # its testing labile
+            if (('lars' in clf.__tags__) and cfg.getboolean('tests', 'labile', default='yes')) \
+                or (not 'lars' in clf.__tags__):
+                self.assertTrue(stats['ACC'] > 1.2 / Nlabels)
             auc = stats['AUC']
             if (Nlabels == 2) or (Nlabels > 2 and auc[0] is not np.nan):
                 mauc = np.min(stats['AUC'])
@@ -695,8 +700,8 @@ def test_confusion_as_node():
     cv = CrossValidation(
         clf, NFoldPartitioner(),
         errorfx=None,
-        postproc=ChainNode((Confusion(labels=ds.UT),
-                            BayesConfusionHypothesis())))
+        postproc=ChainNode([Confusion(labels=ds.UT),
+                            BayesConfusionHypothesis()]))
     res = cv(ds)
     # only two possible hypothesis with two classes
     assert_equals(len(res), 2)
@@ -707,6 +712,31 @@ def test_confusion_as_node():
     # (both classes can be distinguished)
     assert(np.e**res.samples[0,0] < np.e**res.samples[1,0])
 
+    # Let's see how well it would work within the searchlight when we also
+    # would like to store the hypotheses per each voxel
+    # Somewhat an ad-hoc solution for the answer posted on the ML
+    #
+    # run 1d searchlight of radii 0, for that just provide a .fa with coordinates
+    ds.fa['voxel_indices'] = [[0], [1]]
+    # and a custom Node which would collect .sa.hypothesis to place together along
+    # with the posterior probabilities
+    from mvpa2.base.node import Node
+    from mvpa2.measures.searchlight import sphere_searchlight
+
+    class KeepBothPosteriorAndHypothesis(Node):
+        def _call(self, ds):
+            out = np.zeros(1, dtype=object)
+            out[0] = (ds.samples, ds.sa.hypothesis)
+            return out
+    cv.postproc.append(KeepBothPosteriorAndHypothesis())
+    sl = sphere_searchlight(cv, radius=0, nproc=1)
+    res = sl(ds)
+
+    assert_equal(res.shape, (1, 2))
+    assert_equal(len(res.samples[0,0]), 2)
+    assert_equal(res.samples[0,0][0].shape, (2, 2))   # posteriors per 1st SL
+    assert_equal(len(res.samples[0,0][1]), 2)   # 2 of hypotheses
+
 
 def test_bayes_confusion_hyp():
     from mvpa2.clfs.transerror import BayesConfusionHypothesis
@@ -721,7 +751,7 @@ def test_bayes_confusion_hyp():
     skip_if_no_external('scipy')        # uses factorial from scipy.misc
     hyptest = bayes(conf)
     # by default comes with all hypothesis and posterior probs
-    assert_equal(hyptest.shape, (15,2))
+    assert_equal(hyptest.shape, (15, 2))
     assert_array_equal(hyptest.fa.stat, ['log(p(C|H))', 'log(p(H|C))'])
     # check order of hypothesis (coarse)
     assert_array_equal(hyptest.sa.hypothesis[0], [['A', 'B', 'C', 'D']])
@@ -730,21 +760,22 @@ def test_bayes_confusion_hyp():
     # non-log scale
     bayes = BayesConfusionHypothesis(labels_attr='labels', log=False,
                 hypotheses=[[['A', 'B', 'C', 'D']],
-                            [['A', 'C',], ['B', 'D']],
-                            [['A', 'D',], ['B', 'C']],
+                            [['A', 'C'], ['B', 'D']],
+                            [['A', 'D'], ['B', 'C']],
                             [['A'], ['B'], ['C'], ['D']]])
     hyptest = bayes(conf)
     # also with custom hyp the post-probs must add up to 1
-    post_prob = hyptest.samples[:,1]
+    post_prob = hyptest.samples[:, 1]
     assert_almost_equal(np.sum(post_prob), 1)
     # in this particular case ...
     assert(post_prob[3] - np.sum(post_prob[1:3]) < 0.02)
 
 
-def suite():
+def suite():  # pragma: no cover
     return unittest.makeSuite(ErrorsTests)
 
 
-if __name__ == '__main__':
+if __name__ == '__main__':  # pragma: no cover
     import runner
+    runner.run()
 
