@@ -115,6 +115,14 @@ def test_zscore():
     zscore(data, param_est=('targets', [0, 1]))
     assert_array_equal(samples, data.samples)
 
+    # verify that if param_est is set but chunks_attr is None
+    # performs zscoring across entire dataset correctly
+    data = data.copy()
+    data_01 = data.select({'targets': [0, 1]})
+    zscore(data_01, chunks_attr=None)
+    zscore(data, chunks_attr=None, param_est=('targets', [0, 1]))
+    assert_array_equal(data_01.samples, data.select({'targets': [0, 1]}))
+
     # these might be duplicating code above -- but twice is better than nothing
 
     # dataset: mean=2, std=1
@@ -185,3 +193,14 @@ def test_zscore():
 
     assert_array_almost_equal(np.std(ds, axis=0)/np.array(stds),
                               np.std(dsz, axis=0))
+
+def test_zscore_withoutchunks():
+    # just a smoke test to see if all issues of
+    # https://github.com/PyMVPA/PyMVPA/issues/26
+    # are fixed
+    from mvpa2.datasets import Dataset
+    ds = Dataset(np.arange(32).reshape((8,-1)), sa=dict(targets=range(8)))
+    zscore(ds, chunks_attr=None)
+    assert(np.any(ds.samples != np.arange(32).reshape((8,-1))))
+    ds_summary = ds.summary()
+    assert(ds_summary is not None)
\ No newline at end of file
